#!/usr/local/sisis-pap/bin/perl -w
##MH#######################################################################
#                                                                         #
# MOD-NAME     : genInputForSepaAccountConverter.pl      VERSION: 000/002 #
# LONG-NAME    : Eingabedatei fuer den SEPA Access Converter erzeugen     #
# AUTHOR       : F. Dietz / BVB/A                                         #
# TELEPHONE    : +49-(0)89-28638-2896                                     #
# CREATION-DATE: 31-03-11                                                 #
#                                                                         #
###########################################################################
###########################################################################
#                                                                         #
#                                                                         #
# NAME     : genInputForSepaAccountConverter_01.pl                        #
#                                                                         #
# AUTHOR   : Frank Dietz                                                  #
#                                                                         #
#                                                                         #
# FUNCTION : - Eingabedatei:                                              #
#                                                                         #
#                                                                         #
#            - Ausgabedateien:                                            #
#                                                                         #
#              logfile.txt logfile zum Scriptlauf - siehe unten           #
#              das logfile wird ins Ablaufdirectory geschrieben           #
#                                                                         #
#            - .................                                          #
#                                                                         #
#            Preconditions                                                #
#            - db_access.pl ins Ablaufdirectory stellen                   #
#            - .................                                          #
#                                                                         #
#                                                                         #
#            Logfile-Protokollierung:                                     #
#            - Meldungen werden klassifiziert nach Info, Warning, Error   #
#            - alle Error-Meldungen werden zum Scriptende auf stdout      #
#              ausgegeben                                                 #
#                                                                         #
#                                                                         #
# USAGE    : - SSH oeffnen (zum Test auf host = t37pl2, DB = fws)         #
#            - cd /home/sisis/dietz/genInputForSepaAccountConverter       #
#            - perl genInputForSepaAccountConverter.pl                    #
#                                                                         #
#                                                                         #
###########################################################################

# ToDo
# 03042010 : ................

#-----------------------------------------------------------------------
#---------------------- overview subroutines ---------------------------


#  general
#  Subroutine get_os_type
#  Subroutine get_act_date

#  I/O
#  Subroutine signal_array
#  Subroutine signal_hash
#  Subroutine write_file_into_array
#  Subroutine write_array_into_file
#  Subroutine reduce_redundant_entries_in_array
#  Subroutine append_array_at_eof

#  DB access
#  Subroutine get_dbname
#  Subroutine gen_sql_command
#  Subroutine connect_to_database
#  Subroutine execute_sql_command

#  error handling
#  Subroutine set_helptxt
#  Subroutine get_arguments
#  Subroutine show_error_msg
#  Subroutine gen_logfile_string

#  specific subroutines
#  Subroutine clean_blz_kto
#  Subroutine check_blz
#  Subroutine check_kto
#  Subroutine gen_inputlist_for_SEPA_Account_Converter
#  Subroutine gen_empty_strings
#  Subroutine gen_inputlist_for_SEPA_Account_Converter


#-----------------------------------------------------------------------
#---------------------- about script -----------------------------------


my $SCRIPTVERS      = "02";
my $SCRIPTNAME      = "genInputForSepaAccountConverter".".pl";


#-----------------------------------------------------------------------
#---------------------- packages ---------------------------------------


use DBI;
use Tie::RefHash;
#use strict;


#-----------------------------------------------------------------------
#---------------------- require  ---------------------------------------


require "db_access.pl";


#-----------------------------------------------------------------------
#---------------------- config parameter  ------------------------------


my $DBAVAILABLE           = "true";  # DB verfuegbar
my $PRINT_TIMESTAMP       = "false"; # Fehlermeldungen mit Zeitstempel
my $ONLYFORTEST           = "false"; # zusaetzliche Testausgaben
my $LOCALFLAG             = "true";  # Zwischenschritte in (Test)Dateien speichern


#------------------------------------------------------------------------
#---------------------- standard variables ------------------------------


my $INVALID                        = "invalid";
my $OSNAME                         = ();

my @INPUT_ARRAY                    = ();
my @LOG_ARRAY                      = ();
my @RESULT_ARRAY                   = ();
my @ERROR_ARRAY                    = ();
my @SQLRESULT_ARRAY                = ();

my $SQLCOMMAND                     = ();

my $INPUTFILE                      = "inputfile\.txt";
my $LOGFILE                        = "logfile_genInputForSepaAccountConverter\.txt";
my $RESULTFILE                     = "resultfile\.txt";

my $INFO                           = "INFO     : ";
my $WARNING                        = "WARNING  : ";
my $ERROR                          = "ERROR    : ";

my $SEPARATOR                      = "  ---  ";


#-----------------------------------------------------------------------
#---------------------- specific variables -----------------------------


my @ACQ_LIEFLEIT_ARRAY                 = ();
my @SUPPLIER_WITHOUT_ERROR_ARRAY       = ();
my @SEPA_ARRAY                         = ();

my $input_separator                    = ":";
my $blz_string_length                  = 8;
my $kto_string_length                  = 10;

my $tmp_1                              = ();
my $tmp_2                              = ();
my $tmp_1_length                       = 50;
my $tmp_2_length                       = 75;

my $INPUTFILE_CRYSTAL_REPORTS_DATA     = "inputfile_Crystal_Reports\.txt";
my $TRANSFORMED_CRYSTAL_REPORTS_DATA   = "transformed_Crystal_Reports_data\.txt";
my $TRANSFORMED_DB_FORMAT_DATA         = "transformed_db_data\.txt";

my $NUMBER_OF_SUPPLIER_RECORDS         = 0;
my $NUMBER_OF_VALID_SUPPLIER_RECORDS   = 0;
my $NUMBER_OF_INVALID_SUPPLIER_RECORDS = 0;
my $NUMBER_OF_FOREIGN_SUPPLIER_RECORDS = 0;


###############################################################################
#  Subroutine get_os_type
###############################################################################


sub get_os_type{


    # Betriebsumgebung (Windows/UNIX/LINUX) ermitteln
    #
    # Achtung!
    # Bei den temporaeren Dateien $TMPFILE und $TMPERRFILE
    # muessen die Rechte im UNIX/LINUX Betrieb stimmen!
    # Gegebenenfalls ist die umask zu korrigieren:
    # - umask 000 $TMPERRFILE
    # - umask 000 $TMPFILE
    # (das inverse Muster angeben!)


    my $TMPERRFILE   = ();
    my $COMMAND      = ();
    my $RETURNVALUE  = ();

    $TMPERRFILE    = "tmpfile\.txt";
    $COMMAND       = "date /t 2> $TMPERRFILE";
    $RETURNVALUE   = `$COMMAND`;
    #print ("\n\nOS=$RETURNVALUE\n\n");

    $_=$RETURNVALUE;
    chomp $_;

    if (/\./)
    {
       $OSNAME="WIN";
       $DELCOMMAND    = "del $TMPERRFILE";
    }
    else
    {
       $COMMAND="date";
       my $RETURNVALUE=`$COMMAND`;

       $_=$RETURNVALUE;
       if    ( (/MEST/) || (/MET/)) { $OSNAME="UNIX";   }
       elsif ( (/CEST/) || (/CET/)) { $OSNAME="LINUX";  }
       else                         { $OSNAME=$INVALID; }

       $DELCOMMAND    = "rm $TMPERRFILE";
    }
    ### print ("\n\nOS=$OSNAME\n\n\n");

    $RETURNVALUE   = `$DELCOMMAND`;
}


###############################################################################
#  Subroutine get_act_date
###############################################################################


sub get_act_date {


   # Wenn die Subroutine einen Parameter mitbekommt,
   # dann wird neben der Zeit das Datum mit zurueckgegeben


   my @TMPARRAY    = @_;
   my $DATECOMMAND = ();


   &get_os_type();


  if ( "WIN" eq $OSNAME )
  {
     $DATECOMMAND="TIME /T";
     $RETURNVALUE=`$DATECOMMAND`;
  }
  else
  {
     if ( $#TMPARRAY == 0 ) {

        $DATECOMMAND="date '+%H:%M:%S (%m/%d/%y)'";
        $RETURNVALUE=`$DATECOMMAND`;

     } else {

        $DATECOMMAND="date '+%H:%M:%S'";
        $RETURNVALUE=`$DATECOMMAND`;
     }
  }

   chomp  $RETURNVALUE;
   return $RETURNVALUE;
}


###############################################################################
#  Subroutine signal_array
###############################################################################


sub signal_array {


   # es gibt zwei Aufrufsmoeglichkeiten:
   # - signal_array(@ARRAY):
   #   @ARRAY wird ohne Zeilennummerierung ausgegeben
   # - signal_array("withno",@ARRAY):
   #   @ARRAY wird mit Zeilennummerierung ausgegeben
   #
   # --------------------------------------------------------
   #
   # in dieser foreach Konstruktion ist die
   # Laufvariable   = $_
   # Parameterarray = @_
   #
   # --------------------------------------------------------

   @TMPARRAY = @_;

   if (( $#TMPARRAY >= 0   )  &&
       ( $_[0] eq "withno" )) {

      ### zwei Parameter ("withno", Arrayname)
      $WITHNO = ();
      $WITHNO = shift @_;
      $I = 0;
      foreach (@_){

         print("($I) $_\n");
         $I++;
      };

   } else  {

      foreach (@_){ print("$_\n");};
   }
}


###############################################################################
#  Subroutine signal_hash
###############################################################################


sub signal_hash {

   my $ref_hash = shift;
   while (($key, $value) = each(%$ref_hash)){
      print "... key = $key, value = $value\n";
   }
}


##############################################################################
#  Subroutine write_file_into_array
###############################################################################


sub write_file_into_array {



    # die Routine kann entweder mit einem Parameter $_[0] oder
    # mit zwei Parametern $_[0], $_[1] aufgerufen werden
    #
    #
    # Fall 1 - Aufruf mit einem Parameter (mit die):
    # $_[0] = Dateinamen mit vollstaendigem Dateienpfad
    #
    #
    # Fall 2 - Aufruf mit zwei Parametern (ohne die):
    # $_[0] = "WITHOUT_DIE" - das Lesen ueber die wird ausgeschaltet
    # $_[1] = Dateinamen mit vollstaendigem Dateienpfad
    # Kann die Datei nicht geoeffnet werden, dann wird der
    # globale Skalar $RETURNVALUEWITHOUTDIE auf "notavailable" gesetzt.
    #
    #
    # die Datei wird in den Array @INPUT_ARRAY geschrieben


    $RETURNVALUEWITHOUTDIE="available";


    if ($_[0] eq "WITHOUT_DIE"){

       # Datei oeffnen ohne die
       open  FILEHANDLE, "$_[1]" or $RETURNVALUEWITHOUTDIE="notavailable";

    } else {

       # Datei oeffnen mit die
       open  FILEHANDLE, "$_[0]" or die "\n\n... $_[0] not available ($!)\n\n\n";
    }


    $INDEX=0;

    if ($RETURNVALUEWITHOUTDIE eq "available") {

       if ($_[0] eq "WITHOUT_DIE"){
          print ("\n... reading  $_[1] \n\n\n");
       } else {
          print ("\n... reading  $_[0] \n\n\n");
       }


       while ($_=<FILEHANDLE>)
       {
          chomp ($_);
          $INPUT_ARRAY[$INDEX]=$_;
          $INDEX++;
       }
    }

    close FILEHANDLE;

}


###############################################################################
#  Subroutine write_array_into_file
###############################################################################


sub write_array_into_file {


    # diese Routine benoetigt als Eingabeparameter
    # einen Dateinamen und einen Array
    # -> $_[0]  enthaelt den Namen der Zieldatei
    #           mit vollstaendiger Pfadangabe
    # -> $_[1]  enthaelt den zu schreibenden Array


    # Datei oeffnen
    open  OUTPUTHANDLE, "> $_[0]" or die "\n\n... error writing $_[0] ($!)\n\n\n";
    print ("\n... writing  $_[0]\n\n\n");


    $INDEX=0;

    foreach (@_){

       if ($INDEX > 0)
       {
          print( OUTPUTHANDLE "$_\n");
       }
       else
       {
          $INDEX++;
       }
    };

    #Datei schliessen
    close OUTPUTHANDLE;
}


###############################################################################
#  Subroutine reduce_redundant_entries_in_array
###############################################################################


sub reduce_redundant_entries_in_array {


    # Mehrfach-Array-Eintraege werden auf einen reduziert;
    # Algorithmus:
    # Jeden Eintrag als Schluessel eines Hashes benutzen, dann
    # gehen alle gleichen Array-Eintraege auf das gleiche Hashfeld.
    # Der Hash-Wert spielt keine Rolle; es kann ein beliebiges Zeichen
    # Zeichen benutzt werden.
    #
    # $_[0] enthaelt eine Referenz auf den Array
    # $_[1] enthaelt einen String; dieser Parameter ist optional
    #       WITHOUT_SORT : die Sortierung des Ergebnis wird ausgeschaltet
    #            %%%     : das Ergebnis wird sortiert
    # Das bereinigte Array wird zurueckgegeben


    $NUMBEROFPARAMETER  = $#_;
    @UNCLEANEDARRAY = @{$_[0]};
    @CLEANEDARRAY   = ();

    my %RESULTHASH   = ();

    foreach $var ( @UNCLEANEDARRAY ){

       $RESULTHASH{ $var } = "1";
    }

    while (( my $KEY_OF_HASH, my $VALUE_OF_HASH ) = each %RESULTHASH ){

       push ( @CLEANEDARRAY, $KEY_OF_HASH );
    }

    if ( $NUMBEROFPARAMETER == 0 ){
       print "... sorting results\n\n\n";
       @CLEANEDARRAY = sort @CLEANEDARRAY;
    } else {
       print "... sorting results\n\n\n";
    }

    return @CLEANEDARRAY;
}


###############################################################################
#  Subroutine append_array_at_eof
###############################################################################



sub append_array_at_eof {


    # diese Routine benoetigt als Eingabeparameter
    # einen Dateinamen und einen Array
    # -> $_[0]  enthaelt den Namen der Zieldatei
    #           mit vollstaendiger Pfadangabe
    # -> $_[1]  enthaelt den zu schreibenden Array


    # Datei oeffnen
    open  OUTPUTHANDLE, ">> $_[0]" or die "\n\n... error writing $_[0] ($!)\n\n\n";
    ### print ("\n... writing  $_[0]\n\n\n");

    $INDEX=0;

    foreach (@_){

       if ($INDEX > 0)
       {
          print( OUTPUTHANDLE "$_\n");
       }
       else
       {
          $INDEX++;
       }
    };
    #Datei schliessen
    close OUTPUTHANDLE;
}


###############################################################################
#  Subroutine get_dbname
###############################################################################


sub get_dbname {

   # Description
   #
   #

   $COMMAND="echo \$DATABASE";
   my $RETURNVALUE=`$COMMAND`;

   $_=$RETURNVALUE;
   # white-space rausloeschen
   s/\s+//gi;

   # Leerstring-Pruefung
   if (! /^\s*$/ )
   {
      $database=$_;
      print ("... getting name of database ($database)\n\n\n");

   } else {

      $database=$INVALID;
      print ("... invalid value for database \n\n\n");
      exit;
   }
}


###############################################################################
#  Subroutine gen_sql_command
###############################################################################


sub gen_sql_command {


   # Definition verschiedener SQL-Kommandos um Daten aus der aktuellen
   # DB zu holen:
   # - Feldstrukturtabelle auslesen
   # - Blobdaten holen mit definiertem Katkey-Bereich
   # - Blobdaten holen mit definiertem Aktions-Bereich
   #
   # $_[0]  :  Parameter enthaelt eine Option zur Generierung
   #           des sql-Kommandos


   my $sqlcommand  = ();
   my $option      = shift;


   # ------------------------ #
   # Feldstrukturtabelle      #
   # ------------------------ #

   if ( $option eq "getFeldstrukturEntries" ){

      $sqlcommand = q{
        SELECT   fnr, name, kateg, fldtyp
        FROM     sik_fstab
        WHERE    setnr = 1
        ORDER BY fnr
      };

   } elsif ( $option eq "getFeldnrFromFeldstrukturtabelle" ){

      $sqlcommand = q{
        SELECT   fnr,kateg,fldtyp,name,refnr
        FROM     sik_fstab
        WHERE    setnr=1 and fldtyp='V'
        ORDER BY fnr
      };

   # ------------------------ #
   # titel_verbund            #
   # ------------------------ #

   } elsif ( $option eq "getBVNummer" ){

      $sqlcommand = q{
        SELECT verbundid
        FROM   titel_verbund
        WHERE  katkey = ?
      };

   # ------------------------ #
   # titel_daten              #
   # ------------------------ #

   } elsif ( $option eq "test" ){

      $sqlcommand = q{
        SELECT katkey, aktion, nettodaten
        FROM   titel_daten
        WHERE  katkey >= 0 AND katkey <= 10
      };

   } elsif ( $option eq "katkey_all" ) {

      $sqlcommand = q{
        SELECT katkey, aktion, nettodaten
        FROM   titel_daten
      };

   } elsif ( $option eq "katkey_range" ) {

      $sqlcommand = q{
        SELECT katkey, aktion, nettodaten
        FROM   titel_daten
        WHERE  katkey >= ? AND katkey <= ?
      };

   } elsif ( $option eq "katkey_single" ) {

      $sqlcommand = q{
        SELECT katkey, aktion, nettodaten
        FROM   titel_daten
        WHERE  katkey = ?
      };

   # ------------------------ #
   # titel_kette              #
   # ------------------------ #

   } elsif ( $option eq "countDesknrEntries" ){

      $sqlcommand = q{
        SELECT count(*)
        FROM   titel_kette
        WHERE  desknr = ?
      };

   } elsif ( $option eq "getTitelketteEntries" ){

      $sqlcommand = q{
        SELECT *
        FROM   titel_kette
        WHERE  desknr = ?
      };

   # ------------------------ #
   # titel_verweise           #
   # ------------------------ #

   } elsif ( $option eq "getTitelVerweise" ){

      $sqlcommand = q{
        SELECT *
        FROM   titel_verweise
        WHERE  desk = ?
      };

   } elsif ( $option eq "countTitelVerweise" ){

      $sqlcommand = q{
        SELECT count (*)
        FROM   titel_verweise
      };

   } elsif ( $option eq "countFeldnrEntries" ){

      $sqlcommand = q{
        SELECT count (*)
        FROM   titel_verweise
        WHERE  feldnr = ?
      };

   } elsif ( $option eq "getAllTitelVerweise" ){

      $sqlcommand = q{
        SELECT *
        FROM   titel_verweise
      };

   # ------------------------ #
   # per_daten                #
   # ------------------------ #

   } elsif ( $option eq "getBVentries" ) {

      $sqlcommand = q{
        SELECT katkey, id
        FROM   per_daten
        WHERE  id like "BV%"
      };

   } elsif ( $option eq "countBVentries" ) {

      $sqlcommand = q{
        SELECT count(*)
        FROM   per_daten
        WHERE  id like "BV%"
      };

   # ------------------------ #
   # supplier data            #
   # ------------------------ #

   } elsif ( $option eq "getSupplierdata_old" ) {

      $sqlcommand = q{
        SELECT a.nr, a.blz, a.kto, b.lkz
        FROM   acq_liefleit a, acq_adresse b
        WHERE  a.nr = b.nr
      };

   } elsif ( $option eq "getSupplierdata" ) {

      $sqlcommand = q{
        SELECT nr, gruppe, brgroup, geltung, blz, kto
        FROM   sisis.acq_liefleit
      };

   # ------------------------ #
   # invalid parameter        #
   # ------------------------ #


   } else {

      $sqlcommand  = "invalid parameter for sql command\n\n\n";
      exit;
   }

   if ( $ONLYFORTEST eq "true" ){
      print "... sqlcommand = $sqlcommand\n\n\n";
   }
   return $sqlcommand;
}


###############################################################################
#  Subroutine connect_to_database
###############################################################################


sub connect_to_database {

   # Connect to
   #    Datenbankverwaltung --> SYBASE
   #    Datenbank           --> hffsis
   #    Server              --> sokrates
   #
   # - das Kommando ausfuehren, dass in $SQLCOMMAND steht
   # - evt gibt es noch Parameter fuer das SQL-Kommando;
   # - Abfrageergebnissse werden geschrieben nach @SQLRESULTS;


   $ONLYFORTEST = "true";
   if ( $ONLYFORTEST  eq "true" ){
      print ("... running connect_to_database\n\n\n");
   }

   my $counter          = 0;
   my $resultstring     = ();
   my @singlesqlresult  = ();


   # connection to db (DataBaseHandler)
   $dbh = DBI->connect("dbi:Sybase:server=$server;database=$database",
                       $user,
                       $password,
                       { AutoCommit => 1,
                         PrintError => 0,
                         RaiseError => 0
                       }) or die "Fehler bei connect: $DBI::errstr\n";


   # SQL Statement aufbereiten und speichern (Statementhandler)
   $sth = $dbh->prepare( $SQLCOMMAND );
   if ( $ONLYFORTEST  eq "true" ){
      print "... sth = $sth\n\n\n";
   }

   if ( $#_ <  0 ) {


      # ----------------------------------- #
      # SQL Kommando ohne execute-Parameter #
      # ----------------------------------- #

      $sth->execute();

      # alle Abfrage-Ergebnisse in einer Schleife abholen und ausgeben;
      # Ergebnisse werden datensatzweise nach @singlesqlresult geschrieben

      while (@singlesqlresult = $sth->fetchrow_array()) {

         $resultstring = ();
         for ( my $i = 0; $i <= $#singlesqlresult; $i++ ){

            if ( $i == 0 ) { $resultstring = $singlesqlresult[$i];                }
            if ( $i >  0 ) { $resultstring = $resultstring.$SEPARATOR.$singlesqlresult[$i]; }

         }

         push ( @SQLRESULT_ARRAY, $resultstring );
         ### print "... resultstring = $resultstring ($#singlesqlresult)\n";

         $counter ++;
      }
   }


   if ( $#_ >= 0 ) {

      if ( $ONLYFORTEST  eq "true" ){
         print "... running in execute mode ($#_)\n\n\n";
      }

      # ----------------------------------- #
      # SQL Kommando mit execute-Parametern #
      # ----------------------------------- #

      if ( $#_ == 0 ) { $sth->execute( $_[0] );                 }
      if ( $#_ == 1 ) { $sth->execute( $_[0] , $_[1] );         }
      if ( $#_ == 2 ) { $sth->execute( $_[0] , $_[1] , $_[2] ); }

      # alle Abfrage-Ergebnisse in einer Schleife abholen und ausgeben;
      # Ergebnisse werden datensatzweise nach @singlesqlresult geschrieben

      while (@singlesqlresult = $sth->fetchrow_array()) {

         $resultstring = ();
         for ( my $i = 0; $i <= $#singlesqlresult; $i++ ){

            if ( $i == 0 ) { $resultstring = $singlesqlresult[$i];                }
            if ( $i >  0 ) { $resultstring = $resultstring.$SEPARATOR.$singlesqlresult[$i]; }

         }

         push ( @SQLRESULT_ARRAY, $resultstring);
         ### print "... resultstring = $resultstring ($#singlesqlresult)\n";

         $counter ++;
      }
   }


   # ----------------------------------- #
   #                                     #
   # ----------------------------------- #


   # SQL Statement beenden (DB-Verbindung bleibt bestehen)
   $sth->finish();

   # Verbindung zur DB aufheben
   $dbh->disconnect();

   if ( $ONLYFORTEST  eq "true" ){
      print ("\n\n\n");
      print ("... index of highest entry in SQLRESULT_ARRAY is $#SQLRESULT_ARRAY\n\n\n");
      ###&signal_array( @SQLRESULT_ARRAY );
   }
}


###############################################################################
#  Subroutine execute_sql_command
###############################################################################


sub execute_sql_command {


   # - hier erfolgt der Aufruf eines SQL-Kommandos
   # - @SQLRESULT_ARRAY enthaelt das Ergebnis des Aufrufs

   my $msg       = ();
   my $counter   = 0;
   
   if ( $DBAVAILABLE eq "true" ){
   	
      @SQLRESULT_ARRAY = ();
      $SQLCOMMAND=&gen_sql_command( "getSupplierdata" );
      print "... SQLCOMMAND = $SQLCOMMAND\n\n\n";
      &connect_to_database();
      @ACQ_LIEFLEIT_ARRAY = @SQLRESULT_ARRAY;

      &write_array_into_file( $INPUTFILE, @ACQ_LIEFLEIT_ARRAY );
      $counter = $#ACQ_LIEFLEIT_ARRAY + 1;
      
      $msg = "$counter entries found in acq_liefleit";
      print "... $msg\n\n\n";      
      $msg = &gen_logfile_string( $INFO, $msg );
      push ( @LOG_ARRAY, $msg);


   } else {
   	
      $msg = "working without DB access";
      print "... $msg\n\n\n";      
      $msg = &gen_logfile_string( $INFO, $msg );
      push ( @LOG_ARRAY, $msg);

      write_file_into_array($INPUTFILE);
      @ACQ_LIEFLEIT_ARRAY = @INPUT_ARRAY;
      $counter = $#ACQ_LIEFLEIT_ARRAY + 1;
      
      $msg = "$counter entries found in acq_liefleit";
      print "... $msg\n\n\n";      
      $msg = &gen_logfile_string( $INFO, $msg );
      push ( @LOG_ARRAY, $msg);
   }
}


###############################################################################
#  Subroutine gen_script_info
###############################################################################


sub gen_script_info {
	
   my $script_status = shift;
   my $about_script  = ();
   my $act_date      = ();

   $act_date = &get_act_date("WITHDATE");
   if ( $script_status eq "start" ){
      $about_script = "$SCRIPTNAME (Version $SCRIPTVERS) started at $act_date";   	
   } else {
      $about_script = "$SCRIPTNAME (Version $SCRIPTVERS) finished at $act_date";   	   	
   }
   print ("\n\n... $about_script\n\n\n");
   $about_script = $INFO.$about_script;
   push ( @LOG_ARRAY, $about_script );
}


###############################################################################
#  Subroutine set_helptxt
###############################################################################


sub set_helptxt {


   @HELPTXT=(
   " ",
   " ",
   "-h      gibt eine kurze Beschreibung der Argumentliste",
   " ",

   );
}


###############################################################################
#  Subroutine get_arguments
###############################################################################


sub get_arguments {

   # Description
   #
   #

   my @feld = ();

   my $ONLYFORTEST="true";
   if ( $ONLYFORTEST eq "true" ){


      foreach $var (@ARGV){

         $_=$var;
         s/\s+//gi;

         if ( /-h/ ){

            print "\n\n\n";
            &set_helptxt();
            &signal_array( @HELPTXT );
            exit;

         } elsif( /-help/ ) {

            print "\n\n\n";
            &set_helptxt();
            &signal_array( @HELPTXT );
            exit;

         } else    {

           ###print "... used invalid argument: $var\n\n\n";
           ###exit;
         }
      }
   }
}


################################################################################
#  Subroutine show_error_msg
###############################################################################


sub show_error_msg {


   # - es werden alle Meldungen der Klasse Error aus LOG_ARRAY herausgefiltert
   # - Errors werden am Ende des Scripts auf Stdout ausgegeben und signalisiert
   # - Warnings und Infos bleiben unberuecksichtigt

   foreach my $var (@LOG_ARRAY){

      $_=$var;
      if ( /$ERROR/gi ){
         push ( @ERROR_ARRAY, $var);
      }
   }

   if ( $#ERROR_ARRAY >= 0 ){

      ###@ERROR_ARRAY = &reduce_redundant_entries_in_array( \@ERROR_ARRAY, "WITHOUT_SORT" );
      my $counter = $#ERROR_ARRAY + 1;

      print"\n\n";
      print"... error(s) found:\n\n";
      &signal_array( @ERROR_ARRAY );
      print"\n\n";

   } else {

      print"... no errors found\n\n";
   }
}


###############################################################################
#  Subroutine gen_logfile_string
###############################################################################


sub gen_logfile_string {

   # - die subroutine benoetigt gesetzte die Variable $PRINT_TIMESTAMP,
   #   durch die festgelegt wird ob Meldungen einen Zeitstempel erhalten
   #   oder nicht
   # - das Setzen des Zeitstempels kann erhebliche Laufzeit kosten, wenn
   #   es viele Logfile-Eintraege gibt
   # - $msg_type klassifiziert Logfile-Meldungen in drei Gruppen:
   #   Errors    -> werden am Scriptende ausgegeben
   #   Warnings  -> werden am Scriptende nicht ausgegeben
   #   Infos     -> werden am Scriptende nicht ausgegeben

   my $msg_type        = shift;
   my $log_string      = shift;

   if ( $PRINT_TIMESTAMP eq "true" ) {
      my $act_date = &get_act_date("WITHDATE");
      return $act_date."   ".$msg_type.$log_string;
   } else {
      return $msg_type.$log_string;
   }
}


###############################################################################
#  Subroutine show_counters
###############################################################################


sub show_counters {

   my $msg = ();

   print "... running show_counters\n\n\n";

   $msg = "NUMBER_OF_SUPPLIER_RECORDS            = $NUMBER_OF_SUPPLIER_RECORDS";
   $msg = &gen_logfile_string( $INFO, $msg );
   push ( @LOG_ARRAY, $msg );
   print "$msg\n";
   $msg = "NUMBER_OF_VALID_SUPPLIER_RECORDS      = $NUMBER_OF_VALID_SUPPLIER_RECORDS";
   $msg = &gen_logfile_string( $INFO, $msg );
   push ( @LOG_ARRAY, $msg );
   print "$msg\n";
   $msg = "NUMBER_OF_INVALID_SUPPLIER_RECORDS    = $NUMBER_OF_INVALID_SUPPLIER_RECORDS";
   $msg = &gen_logfile_string( $INFO, $msg );
   push ( @LOG_ARRAY, $msg );
   print "$msg\n";

   print "\n\n";
}



###############################################################################
#  Subroutine clean_blz_kto
###############################################################################


sub clean_blz_kto {

   # Bereinigung von blz oder kto durch loeschen von
   # Leerzeichen, slash, Punkt und Bindestrich

   $_ = shift;
   s/ //gi;
   s/\///gi;
   s/-//gi;
   s/\.//gi;
   return $_;
}


###############################################################################
#  Subroutine check_blz
###############################################################################


sub check_blz {

   # - blz wird einer Formatpruefungen unterzogen; falls notwendig wird 
   #   Formataenderung durchgefuehrt
   # - wenn ein ungueltiges Feld erkannt wird, dann wird dieses mit einem
   #   Prefix zur naeheren Spezifizierung des Fehlers zurueckgeschickt
   
   my $blz = shift;

   # Bereinigung
   $blz = &clean_blz_kto ($blz);

   # ggf. linksbuendig "0" auffuellen
   if ( length($blz) > $blz_string_length ){

      $blz = "blz_zu_lang ".$blz;

   } elsif ( length($blz) < $blz_string_length ) {

      if ( length($blz) == 0 ){
         $field_value = "blz_not_set ";
      } else {
         my $diff = $blz_string_length - length($blz);
         for ( my $counter = 0; $counter < $diff; $counter ++ ){
            $blz = "0".$blz;
         }
      }
   }
   return $blz;
}


###############################################################################
#  Subroutine check_kto
###############################################################################


sub check_kto {

   # - blz wird einer Formatpruefungen unterzogen; falls notwendig wird 
   #   Formataenderung durchgefuehrt
   # - wenn ein ungueltiges Feld erkannt wird, dann wird dieses mit einem
   #   Prefix zur naeheren Spezifizierung des Fehlers zurueckgeschickt
   
   my $kto = shift;

   # Bereinigung
   $kto = &clean_blz_kto ($kto);

   # ggf. linksbuendig "0" auffuellen
   if ( length($kto) > $kto_string_length ){

      $kto = "kto_zu_lang ".$kto;

   } elsif ( length($kto) < $kto_string_length ) {

      if ( length($kto) == 0 ){
         $field_value = "kto_not_set ";
      } else {
         my $diff = $kto_string_length - length($kto);
         for ( my $counter = 0; $counter < $diff; $counter ++ ){
            $kto = "0".$kto;
         }
      }
   }
   return $kto;
}


###############################################################################
#  Subroutine gen_inputlist_for_SEPA_Account_Converter
###############################################################################


sub gen_inputlist_for_SEPA_Account_Converter
 {

   # - ungueltige records ermitteln, aussortieren und als Fehler signalisieren;
   #   Verfahren: 
   #   ungueltige records erhalten einen "invalid" String und werden bei
   #   der Weiterverarbeitung daran erkannt und aussortiert

   print ("... running gen_SEPA_account_converter_inputlist\n\n\n");

   my $error_reason = ();
   my $blz_kto_sep  = " ";
   my $msg          = ();
   my $tmp_string   = ();
   my $nr           = ();
   my $gruppe       = ();
   my $brgroup      = ();
   my $geltung      = ();
   my $blz          = ();
   my $kto          = ();
   my $check_result = ();

   foreach $var (@ACQ_LIEFLEIT_ARRAY){

      $NUMBER_OF_SUPPLIER_RECORDS ++;
      
      $error_reason = "none";
      $tmp_string   = ();
      $msg          = ();
      
      # Leerstring-Pruefung
      $_= $var;
      if (! /^\s*$/ )
      {
         my @feld_1 = ();
         @feld_1    = split /$SEPARATOR/, $var;
         
         if ($#feld_1 >= 5){

            $nr       = $feld_1[0];
            $gruppe   = $feld_1[1];
            $brgroup  = $feld_1[2];
            $geltung  = $feld_1[3];
            $blz      = $feld_1[4];
            $kto      = $feld_1[5];

            # --------------------------------- #
            # format and check blz              #
            # --------------------------------- #
            
            $blz = &check_blz($blz);
            
            my @blz_felder = ();
            @blz_felder    = split /$blz_kto_sep/, $blz;
            $blz           = $blz_felder[$#blz_felder];
            
            if ( $#blz_felder > 0 ){      	
               $error_reason = $blz_felder[0]." ($blz)";
            }
            
            # --------------------------------- #
            # format and check kto              #
            # --------------------------------- #
            
            $kto = &check_kto( $kto);
            
            my @kto_felder = ();
            @kto_felder    = split /$blz_kto_sep/, $kto;
            $kto           = $kto_felder[$#kto_felder];
            
            if ( $#kto_felder > 0 ){      	
               if ( $error_reason ne "none" ){
                  $error_reason = $error_reason."  ,  $kto_felder[0] ($kto)"	
               } else {
                  $error_reason = $kto_felder[0]." ($kto)";
               }
            }
                                    
            # --------------------------------- #
            # blz_kto check                     #
            # --------------------------------- #

            if ( ! defined($blz) ){
               $error_reason = "invalid blz";
            }
            if ( ! defined($kto) ){
               if ( defined($error_reason) ){
                  $error_reason = $error_reason." invalid kto";
               } else {
                 $error_reason = "invalid kto";
               }
            }
            
            $_= $feld_1[4].$feld_1[5];
            if ( /^\s*$/ ){
               $error_reason = "empty blz and kto";
            }
            
            # --------------------------------- #
            # gen SUPPLIER_WITHOUT_ERROR_ARRAY  #
            # --------------------------------- #
            
            if ( $error_reason ne "none" ){
            	
               $msg = &gen_logfile_string( $ERROR, "(1) $error_reason" );
               push ( @LOG_ARRAY, $msg);
               $msg = &gen_logfile_string( $ERROR, "(2) $var" );
               push ( @LOG_ARRAY, $msg);
               push ( @LOG_ARRAY, " " );
            	
               $NUMBER_OF_INVALID_SUPPLIER_RECORDS ++;

            } else {            	
            	
               $tmp_string = $nr.$SEPARATOR;
               $tmp_string = $tmp_string.$gruppe.$SEPARATOR;
               $tmp_string = $tmp_string.$brgroup.$SEPARATOR;
               $tmp_string = $tmp_string.$geltung.$SEPARATOR;
               $tmp_string = $tmp_string.$blz.$SEPARATOR;
               $tmp_string = $tmp_string.$kto;
               	
               $NUMBER_OF_VALID_SUPPLIER_RECORDS ++;               
               push ( @SUPPLIER_WITHOUT_ERROR_ARRAY, $tmp_string );
            }
            
         } else {

            $msg = &gen_logfile_string( $ERROR, "(1) invalid number of fields" );
            push ( @LOG_ARRAY, $msg);
            $msg = &gen_logfile_string( $ERROR, "(2) $var" );
            push ( @LOG_ARRAY, $msg);
            push ( @LOG_ARRAY, " " );
            	
            $NUMBER_OF_INVALID_SUPPLIER_RECORDS ++;
         }                        
      }
   }
}


###############################################################################
#  Subroutine gen_empty_strings
###############################################################################


sub gen_empty_strings {

   my $counter = 0;

   for ( $counter = 0; $counter < $tmp_1_length; $counter ++ ){
             $tmp_1 = $tmp_1." ";
   }
   for ( $counter = 0; $counter < $tmp_2_length; $counter ++ ){
      $tmp_2 = $tmp_2." ";
   }
}


###############################################################################
#  Subroutine gen_inputlist_for_SEPA_Account_Converter
###############################################################################


sub write_inputlist_for_SEPA_Account_Converter {


   # - Zeilen in ein temporaeres array schreiben;
   # - eine Zeile in der Ergebnisdatei ist eine Textzeile mit:
   #   Laenderkennzeichen      Buchstabe 1-2      - aus feld[3]
   #   Bankleitzahl            Buchstabe 53-60    - aus feld[1]
   #   Kontonummer             Buchstabe 61-70    - aus feld[2]
   #   letztes Zeichen (blank) Index 145
   # - zwischen den 3 Eintraegen werden blanks gesetzt
   # - das Laenderkennzeichen wird jetzt immer auf DE gesetzt;
   #   damit werden auch IBAN und BIC von auslaendischen Lieferanten
   #   erreicht, die Kunde deutscher Banken sind

   my @feld           = ();
   my $result_entry   = ();
   my $msg            = ();
   my $blz            = ();
   my $kto            = ();
   my $lkz            = ();

   foreach $var (@SUPPLIER_WITHOUT_ERROR_ARRAY){

      ### only for test
      ### print "SUPPLIER_WITHOUT_ERROR_ARRAY :   $var\n"; 
        	
      $result_entry = ();
      @feld = ();
      @feld = split /$SEPARATOR/, $var;
      
      $lkz = "DE"; 
      $blz = $feld[4];
      $kto = $feld[5];
      
      $result_entry = $lkz.$tmp_1.$blz.$kto.$tmp_2;
      push( @SEPA_ARRAY, $result_entry);
   }
   &write_array_into_file( $RESULTFILE , @SEPA_ARRAY );
}


###############################################################################
#  main part
###############################################################################


&gen_script_info("start");
&write_array_into_file( $LOGFILE , "" );


# --------------------- #
# running script        #
# --------------------- #


if ($DBAVAILABLE eq "true"){
   &get_dbname();
}


&execute_sql_command();

&gen_inputlist_for_SEPA_Account_Converter();

&gen_empty_strings();

&write_inputlist_for_SEPA_Account_Converter();


if ( $LOCALFLAG eq "true" ){

   # Kontrolldateien ins Ablaufdirectory #
   &write_array_into_file( "ACQ_LIEFLEIT_ENTRIES", @SQLRESULT_ARRAY );
   &write_array_into_file( "SUPPLIER_WITHOUT_ERROR", @SUPPLIER_WITHOUT_ERROR_ARRAY );  
}

&show_error_msg();

&show_counters();


# --------------------- #
# avoid warnings        #
# --------------------- #


$INVALID        = ();
$HELPTXT        = ();
$ONLYFORTEST    = ();
$INPUTFILE      = ();
@ERROR_ARRAY    = ();
@RESULT_ARRAY   = ();

$password       = ();
$user           = ();
$server         = ();



###############################################################################
#
###############################################################################


&gen_script_info("end");
&write_array_into_file( $LOGFILE , @LOG_ARRAY );


######################################################################## ENDE #


##HOC######################################################################
#                                                                         #
# HISTORY OF CHANGE:                                         VARIANT: 000 #
#                                                                         #
#----+------+----------------------------------------------+-------+------#
#    | DATE | CHANGES                                      | AUTH  | CNO  #
#    |ddmmyy|                                              |       |      #
#----+------+----------------------------------------------+-------+------#
# 001|131111| Initial creation                             |FDIETZ | 0000 #
#----+------+----------------------------------------------+-------+------#
# 002|191211| lkz Bewertung ausgebaut                      |FDIETZ | 0000 #
###########################################################################








