#!/usr/local/sisis-pap/bin/perl -w
##MH#######################################################################
#                                                                         #
# MOD-NAME     : setIbanBic.pl                VERSION: 000/003 - 07/10/13 #
# LONG-NAME    : IBAN und BIC in die Lieferantendaten schreiben           #
# AUTHOR       : F. Dietz / BVB/A                                         #
# TELEPHONE    : +49-(0)89-28638-2932                                     #
# CREATION-DATE: 31-03-11                                                 #
#                                                                         #
###########################################################################
###########################################################################
#                                                                         #
#                                                                         #
# NAME     : setIbanBic.pl                                                #
#                                                                         #
# AUTHOR   : Frank Dietz                                                  #
#                                                                         #
#                                                                         #
# FUNCTION : - Eingabedatei:                                              #
#                                                                         #
#                                                                         #
#            - Ausgabedateien:                                            #
#                                                                         #
#              logfile.txt logfile zum Scriptlauf - siehe unten           #
#              das logfile wird ins Ablaufdirectory geschrieben           #
#                                                                         #
#            - .................                                          #
#                                                                         #
#            Preconditions                                                #
#            - db_access.pl ins Ablaufdirectory stellen                   #
#            - .................                                          #
#                                                                         #
#                                                                         #
#            Logfile-Protokollierung:                                     #
#            - Meldungen werden klassifiziert nach Info, Warning, Error   #
#            - alle Error-Meldungen werden zum Scriptende auf stdout      #
#              ausgegeben                                                 #
#                                                                         #
#                                                                         #
# USAGE    : - SSH oeffnen (zum Test auf host = t37pl2, DB = fws)         #
#            - cd /home/sisis/dietz/setIbanBic                            #
#            - perl setIbanBic.pl                                         #
#                                                                         #
#                                                                         #
###########################################################################

# ToDo
# 03042010 : ................

#-----------------------------------------------------------------------
#---------------------- overview subroutines ---------------------------


#  general
#  Subroutine get_os_type
#  Subroutine get_act_date

#  I/O
#  Subroutine signal_array
#  Subroutine signal_hash
#  Subroutine write_file_into_array
#  Subroutine write_array_into_file
#  Subroutine reduce_redundant_entries_in_array
#  Subroutine append_array_at_eof

#  DB access
#  Subroutine get_dbname
#  Subroutine gen_where_condition
#  Subroutine gen_sql_command
#  Subroutine connect_to_database
#  Subroutine execute_sql_command

#  error handling
#  Subroutine gen_script_info
#  Subroutine show_DB_availability
#  Subroutine set_helptxt
#  Subroutine get_arguments
#  Subroutine show_error_msg
#  Subroutine gen_logfile_string
#  Subroutine show_counters

#  specific subroutines
#  Subroutine clean_blz_kto
#  Subroutine count_IBAN_BIC_records
#  Subroutine show_counters
#  Subroutine count_error_cases
#  Subroutine show_IBANRUECK_error_statistic
#  Subroutine check_IBANRUECK
#  Subroutine expand_IBANRUECK_LOGFILE_MSG_ARRAY
#  Subroutine read_acq_liefleit_entries
#  Subroutine gen_blz_kto_string
#  Subroutine format_acq_liefleit_array
#  Subroutine search_blz_kto_string
#  Subroutine set_IBAN_BIC
#  Subroutine write_redundancies_into_logfile
#  Subroutine write_blz_kto_to_iban_bic


#-----------------------------------------------------------------------
#---------------------- about script -----------------------------------


my $SCRIPTVERS      = "03";
my $SCRIPTNAME      = "setIbanBic\.pl";


#-----------------------------------------------------------------------
#---------------------- packages ---------------------------------------


use DBI;
use Tie::RefHash;
#use strict;


#-----------------------------------------------------------------------
#---------------------- require  ---------------------------------------


require "db_access.pl";


#-----------------------------------------------------------------------
#---------------------- config parameter  ------------------------------



my $OVERWRITE_IBAN_BIC_FLAG  = "false"; # IBAN-BIC Eintraege ueberschreiben
my $DBAVAILABLE              = "true"; # DB verfuegbar
my $PRINT_TIMESTAMP          = "false"; # Fehlermeldungen mit Zeitstempel
my $ONLYFORTEST              = "false"; # zusaetzliche Testausgaben
my $LOCALFLAG                = "true";  # Zwischenschritte in (Test)Dateien speichern
my $DIVISOR                  = 10;      # Fortschrittsanzeige
my $COPY_BLZ_KTO_TO_IBAN_BIC = "true";  # Eintraege aus blz und kto nach iban, bic kopieren
my $CLEAN_BLZ_KTO            = "true";  # Bereinigung von blz und kto vor dem kopieren


#------------------------------------------------------------------------
#---------------------- standard variables ------------------------------


my $INVALID                        = "invalid";
my $OSNAME                         = ();

my @INPUT_ARRAY                    = ();
my @LOG_ARRAY                      = ();
my @RESULT_ARRAY                   = ();
my @ERROR_ARRAY                    = ();
my @REDUNDANT_ENTRIES_ARRAY        = ();
my @SQLRESULT_ARRAY                = ();

my $SQLCOMMAND                     = ();

my $INPUTFILE                      = "inputfile\.txt";
my $LOGFILE                        = "logfile_setIbanBic\.txt";
my $RESULTFILE                     = "resultfile\.txt";

my $INFO                           = "INFO     : ";
my $WARNING                        = "WARNING  : ";
my $ERROR                          = "ERROR    : ";
my $EMPTY                          = "           ";

my $SEPARATOR                      = "  ---  ";


#-----------------------------------------------------------------------
#---------------------- specific variables -----------------------------


my @IBANRUECK_ARRAY                                = ();
my $IBANRUECK_FILE                                 = "IBANRUECK";
my @IBANRUECK_WITHOUT_REDUNDANCE_ARRAY             = ();
my @IBANRUECK_OK_STATUS_ARRAY                      = ();
my @IBANRUECK_LOGFILE_MSG_ARRAY                    = ();

my @ACQ_LIEFLEIT_ARRAY                             = ();
my $ACQ_LIEFLEIT_FILE                              = "acq_liefleit_entries\.txt";
my @ACQ_LIEFLEIT_WITHOUT_REDUNDANCE_ARRAY          = ();
my @ACQ_LIEFLEIT_FORMATED_ARRAY                    = ();
my @ACQ_LIEFLEIT_INVALID_BLZ_KTO_ARRAY             = ();
my @ACQ_LIEFLEIT_ALL_HITS_ARRAY                    = ();
my @ACQ_LIEFLEIT_SINGLE_HITS_ARRAY                 = ();
my @ACQ_LIEFLEIT_AFTER_UPDATE_ARRAY                = ();

my $NUM_IBANRUECK_ENTRIES                          = 0;
my $NUM_IBANRUECK_ENTRIES_WITHOUT_REDUNDANCE       = 0;
my $NUM_IBANRUECK_OK_STATUS                        = 0;
my $NUM_IBANRUECK_NOT_OK_STATUS                    = 0;

my $NUM_ACQ_LIEFLEIT_ENTRIES                       = 0;
my $NUM_ACQ_LIEFLEIT_ENTRIES_WITHOUT_REDUNDANCE    = 0;
my $NUM_ACQ_LIEFLEIT_ENTRIES_VALID_FORMAT          = 0;
my $NUM_ACQ_LIEFLEIT_ENTRIES_INVALID_FORMAT        = 0;
my $NUM_ACQ_LIEFLEIT_SEPA_RESULTS                  = 0;
my $NUM_ACQ_LIEFLEIT_COPY_BLZ_KTO                  = 0;
my $NUM_ACQ_LIEFLEIT_REDUNDANCIES                  = 0;
my $NUM_ACQ_LIEFLEIT_BLZ_KTO_EMPTY                 = 0;

my $NUM_BLZ_KTO_STRING_FOUND                       = 0;
my $NUM_BLZ_KTO_STRING_NOT_FOUND                   = 0;

my $IBAN_BIC_SET_BEFORE_UPDATE                     = 0;
my $IBAN_BIC_NOT_SET_BEFORE_UPDATE                 = 0;

my $IBAN_BIC_SET_AFTER_UPDATE                      = 0;
my $IBAN_BIC_NOT_SET_AFTER_UPDATE                  = 0;

my %ERROR_HASH                                     = ();


###############################################################################
#  Subroutine get_os_type
###############################################################################


sub get_os_type{


    # Betriebsumgebung (Windows/UNIX/LINUX) ermitteln
    #
    # Achtung!
    # Bei den temporaeren Dateien $TMPFILE und $TMPERRFILE
    # muessen die Rechte im UNIX/LINUX Betrieb stimmen!
    # Gegebenenfalls ist die umask zu korrigieren:
    # - umask 000 $TMPERRFILE
    # - umask 000 $TMPFILE
    # (das inverse Muster angeben!)


    my $TMPERRFILE   = ();
    my $COMMAND      = ();
    my $RETURNVALUE  = ();

    $TMPERRFILE    = "tmpfile\.txt";
    $COMMAND       = "date /t 2> $TMPERRFILE";
    $RETURNVALUE   = `$COMMAND`;
    #print ("\n\nOS=$RETURNVALUE\n\n");

    $_=$RETURNVALUE;
    chomp $_;

    if (/\./)
    {
       $OSNAME="WIN";
       $DELCOMMAND    = "del $TMPERRFILE";
    }
    else
    {
       $COMMAND="date";
       my $RETURNVALUE=`$COMMAND`;

       $_=$RETURNVALUE;
       if    ( (/MEST/) || (/MET/)) { $OSNAME="UNIX";   }
       elsif ( (/CEST/) || (/CET/)) { $OSNAME="LINUX";  }
       else                         { $OSNAME=$INVALID; }

       $DELCOMMAND    = "rm $TMPERRFILE";
    }
    ###print ("\n\nOS=$OSNAME\n\n\n");

    $RETURNVALUE   = `$DELCOMMAND`;
}


###############################################################################
#  Subroutine get_act_date
###############################################################################


sub get_act_date {


   # Wenn die Subroutine einen Parameter mitbekommt,
   # dann wird neben der Zeit das Datum mit zurueckgegeben


   my @TMPARRAY    = @_;
   my $DATECOMMAND = ();


   &get_os_type();


  if ( "WIN" eq $OSNAME )
  {
     $DATECOMMAND="TIME /T";
     $RETURNVALUE=`$DATECOMMAND`;
  }
  else
  {
     if ( $#TMPARRAY == 0 ) {

        $DATECOMMAND="date '+%H:%M:%S (%m/%d/%y)'";
        $RETURNVALUE=`$DATECOMMAND`;

     } else {

        $DATECOMMAND="date '+%H:%M:%S'";
        $RETURNVALUE=`$DATECOMMAND`;
     }
  }

   chomp  $RETURNVALUE;
   return $RETURNVALUE;
}


###############################################################################
#  Subroutine signal_array
###############################################################################


sub signal_array {


   # es gibt zwei Aufrufsmoeglichkeiten:
   # - signal_array(@ARRAY):
   #   @ARRAY wird ohne Zeilennummerierung ausgegeben
   # - signal_array("withno",@ARRAY):
   #   @ARRAY wird mit Zeilennummerierung ausgegeben
   #
   # --------------------------------------------------------
   #
   # in dieser foreach Konstruktion ist die
   # Laufvariable   = $_
   # Parameterarray = @_
   #
   # --------------------------------------------------------

   @TMPARRAY = @_;

   if (( $#TMPARRAY >= 0   )  &&
       ( $_[0] eq "withno" )) {

      ### zwei Parameter ("withno", Arrayname)
      $WITHNO = ();
      $WITHNO = shift @_;
      $I = 0;
      foreach (@_){

         print("($I) $_\n");
         $I++;
      };

   } else  {

      ### ein  Parameter (Arrayname)
      foreach (@_){ print("$_\n");};
   }
}


###############################################################################
#  Subroutine signal_hash
###############################################################################


sub signal_hash {

   my $ref_hash = shift;
   while (($key, $value) = each(%$ref_hash)){
      print "... key = $key, value = $value\n";
   }
}


##############################################################################
#  Subroutine write_file_into_array
###############################################################################


sub write_file_into_array {



    # die Routine kann entweder mit einem Parameter $_[0] oder
    # mit zwei Parametern $_[0], $_[1] aufgerufen werden
    #
    #
    # Fall 1 - Aufruf mit einem Parameter (mit die):
    # $_[0] = Dateinamen mit vollstaendigem Dateienpfad
    #
    #
    # Fall 2 - Aufruf mit zwei Parametern (ohne die):
    # $_[0] = "WITHOUT_DIE" - das Lesen ueber die wird ausgeschaltet
    # $_[1] = Dateinamen mit vollstaendigem Dateienpfad
    # Kann die Datei nicht geoeffnet werden, dann wird der
    # globale Skalar $RETURNVALUEWITHOUTDIE auf "notavailable" gesetzt.
    #
    #
    # die Datei wird in den Array @INPUT_ARRAY geschrieben



    $RETURNVALUEWITHOUTDIE="available";



    if ($_[0] eq "WITHOUT_DIE"){

       # Datei oeffnen ohne die
       open  FILEHANDLE, "$_[1]" or $RETURNVALUEWITHOUTDIE="notavailable";

    } else {

       # Datei oeffnen mit die
       open  FILEHANDLE, "$_[0]" or die "\n\n... $_[0] not available ($!)\n\n\n";
    }


    $INDEX=0;

    if ($RETURNVALUEWITHOUTDIE eq "available") {

       if ($_[0] eq "WITHOUT_DIE"){
          print ("\n... reading  $_[1] \n\n\n");
       } else {
          print ("\n... reading  $_[0] \n\n\n");
       }


       while ($_=<FILEHANDLE>)
       {
          chomp ($_);
          $INPUT_ARRAY[$INDEX]=$_;
          $INDEX++;
       }
    }

    close FILEHANDLE;

}


###############################################################################
#  Subroutine write_array_into_file
###############################################################################


sub write_array_into_file {

    # diese Routine benoetigt als Eingabeparameter
    # einen Dateinamen und einen Array
    # -> $_[0]  enthaelt den Namen der Zieldatei
    #           mit vollstaendiger Pfadangabe
    # -> $_[1]  enthaelt den zu schreibenden Array


    my $INDEX=0;

    # Datei oeffnen
    open  OUTPUTHANDLE, "> $_[0]" or die "\n\n... error writing $_[0] ($!)\n\n\n";
    print ("\n... writing  $_[0]\n\n\n");

    foreach (@_){

       if ($INDEX > 0){
          print( OUTPUTHANDLE "$_\n");
       } else {
          $INDEX++;
       }
    }
    #Datei schliessen
    close OUTPUTHANDLE;
}


###############################################################################
#  Subroutine reduce_redundant_entries_in_array
###############################################################################


sub reduce_redundant_entries_in_array {


    # Mehrfach-Array-Eintraege werden auf einen reduziert;
    # Algorithmus:
    # Jeden Eintrag als Schluessel eines Hashes benutzen, dann
    # gehen alle gleichen Array-Eintraege auf das gleiche Hashfeld.
    # Der Hash-Wert wird inkrementiert und dient dem Aufsammeln von Redundanzen.
    #
    # $_[0] enthaelt eine Referenz auf den Array
    # $_[1] enthaelt einen String; dieser Parameter ist optional
    #       WITHOUT_SORT         : die Sortierung des Ergebnis wird ausgeschaltet
    #       COLLECT_REDUNDANCIES : Redundanzen werden in @REDUNDANT_ENTRIES_ARRAY gesammelt
    #       %%%                  : das Ergebnis wird sortiert
    # Das bereinigte Array wird zurueckgegeben


    my $NUMBEROFPARAMETER  = $#_;
    my @UNCLEANEDARRAY     = @{$_[0]};
    my @CLEANEDARRAY       = ();
    my %RESULTHASH         = ();

    @REDUNDANT_ENTRIES_ARRAY = ();

    foreach $var ( @UNCLEANEDARRAY ){

       if ( exists $RESULTHASH{$var}){
          $RESULTHASH{$var} ++;
          push (@REDUNDANT_ENTRIES_ARRAY, $var);
       } else {
          $RESULTHASH{$var} = 1;
       }
    }

    while (( my $KEY_OF_HASH, my $VALUE_OF_HASH ) = each %RESULTHASH ){

       push ( @CLEANEDARRAY, $KEY_OF_HASH );
    }

    if ( $NUMBEROFPARAMETER == 0 ){
       print "... sorting results\n\n\n";
       @CLEANEDARRAY = sort @CLEANEDARRAY;
    } else {
       if ( $_[1] eq "WITHOUT_SORT" ){
          print "... results without sort\n\n\n";
       } elsif ( $_[1] eq "COLLECT_REDUNDANCIES" ){
          print "... collecting redundancies\n\n\n";
          &write_redundancies_into_logfile( \@REDUNDANT_ENTRIES_ARRAY );
       } else {
          ;;
       }
    }
    return @CLEANEDARRAY;
}


###############################################################################
#  Subroutine append_array_at_eof
###############################################################################



sub append_array_at_eof {


    # diese Routine benoetigt als Eingabeparameter
    # einen Dateinamen und einen Array
    # -> $_[0]  enthaelt den Namen der Zieldatei
    #           mit vollstaendiger Pfadangabe
    # -> $_[1]  enthaelt den zu schreibenden Array


    # Datei oeffnen
    open  OUTPUTHANDLE, ">> $_[0]" or die "\n\n... error writing $_[0] ($!)\n\n\n";
    ### print ("\n... writing  $_[0]\n\n\n");

    $INDEX=0;

    foreach (@_){

       if ($INDEX > 0)
       {
          print( OUTPUTHANDLE "$_\n");
       }
       else
       {
          $INDEX++;
       }
    };
    #Datei schliessen
    close OUTPUTHANDLE;
}


###############################################################################
#  Subroutine get_dbname
###############################################################################


sub get_dbname {

   # Description
   # DB-Name aus der Umgebungsvariablen auslesen

   if ($DBAVAILABLE eq "true"){

      $COMMAND="echo \$DATABASE";
      my $RETURNVALUE=`$COMMAND`;

      $_=$RETURNVALUE;
      # white-space rausloeschen
      s/\s+//gi;

      # Leerstring-Pruefung
      if (! /^\s*$/ )
      {
         $database=$_;
         print ("... getting name of database ($database)\n\n\n");

      } else {

         $database=$INVALID;
         print ("... invalid value for database \n\n\n");
         exit;
      }
   }
}


###############################################################################
#  Subroutine gen_where_condition
###############################################################################


sub gen_where_condition {

   my $condition = ();
   my $nr              = shift;
   my $gruppe          = shift;
   my $brgroup         = shift;
   my $geltung         = shift;
   my $and             = " and ";

   $condition = "nr=".$nr.$and."gruppe=".$gruppe.$and."brgroup=".$brgroup.$and;
   $condition = $condition."geltung=\"".$geltung."\"";
   return $condition;
}


###############################################################################
#  Subroutine gen_sql_command
###############################################################################


sub gen_sql_command {


   # Definition verschiedener SQL-Kommandos um Daten aus der aktuellen
   # DB zu holen:
   # - Feldstrukturtabelle auslesen
   # - Blobdaten holen mit definiertem Katkey-Bereich
   # - Blobdaten holen mit definiertem Aktions-Bereich
   #
   # $_[0]  :  Parameter enthaelt eine Option zur Generierung
   #           des sql-Kommandos


   my $sqlcommand  = ();
   my $option      = shift;


   # ------------------------ #
   # Feldstrukturtabelle      #
   # ------------------------ #

   if ( $option eq "getFeldstrukturEntries" ){

      $sqlcommand = q{
        SELECT   fnr, name, kateg, fldtyp
        FROM     sik_fstab
        WHERE    setnr = 1
        ORDER BY fnr
      };

   } elsif ( $option eq "getFeldnrFromFeldstrukturtabelle" ){

      $sqlcommand = q{
        SELECT   fnr,kateg,fldtyp,name,refnr
        FROM     sik_fstab
        WHERE    setnr=1 and fldtyp='V'
        ORDER BY fnr
      };

   # ------------------------ #
   # titel_verbund            #
   # ------------------------ #

   } elsif ( $option eq "getBVNummer" ){

      $sqlcommand = q{
        SELECT verbundid
        FROM   titel_verbund
        WHERE  katkey = ?
      };

   # ------------------------ #
   # titel_daten              #
   # ------------------------ #

   } elsif ( $option eq "test" ){

      $sqlcommand = q{
        SELECT katkey, aktion, nettodaten
        FROM   titel_daten
        WHERE  katkey >= 0 AND katkey <= 10
      };

   } elsif ( $option eq "katkey_all" ) {

      $sqlcommand = q{
        SELECT katkey, aktion, nettodaten
        FROM   titel_daten
      };

   } elsif ( $option eq "katkey_range" ) {

      $sqlcommand = q{
        SELECT katkey, aktion, nettodaten
        FROM   titel_daten
        WHERE  katkey >= ? AND katkey <= ?
      };

   } elsif ( $option eq "katkey_single" ) {

      $sqlcommand = q{
        SELECT katkey, aktion, nettodaten
        FROM   titel_daten
        WHERE  katkey = ?
      };

   # ------------------------ #
   # titel_kette              #
   # ------------------------ #

   } elsif ( $option eq "countDesknrEntries" ){

      $sqlcommand = q{
        SELECT count(*)
        FROM   titel_kette
        WHERE  desknr = ?
      };

   } elsif ( $option eq "getTitelketteEntries" ){

      $sqlcommand = q{
        SELECT *
        FROM   titel_kette
        WHERE  desknr = ?
      };

   # ------------------------ #
   # titel_verweise           #
   # ------------------------ #

   } elsif ( $option eq "getTitelVerweise" ){

      $sqlcommand = q{
        SELECT *
        FROM   titel_verweise
        WHERE  desk = ?
      };

   } elsif ( $option eq "countTitelVerweise" ){

      $sqlcommand = q{
        SELECT count (*)
        FROM   titel_verweise
      };

   } elsif ( $option eq "countFeldnrEntries" ){

      $sqlcommand = q{
        SELECT count (*)
        FROM   titel_verweise
        WHERE  feldnr = ?
      };

   } elsif ( $option eq "getAllTitelVerweise" ){

      $sqlcommand = q{
        SELECT *
        FROM   titel_verweise
      };

   # ------------------------ #
   # per_daten                #
   # ------------------------ #

   } elsif ( $option eq "getBVentries" ) {

      $sqlcommand = q{
        SELECT katkey, id
        FROM   per_daten
        where id like "BV%"
      };

   } elsif ( $option eq "countBVentries" ) {

      $sqlcommand = q{
        SELECT count(*)
        FROM   per_daten
        where id like "BV%"
      };

   # ------------------------ #
   # supplier data            #
   # ------------------------ #

   } elsif ( $option eq "get_Supplierdata_for_sepa" ) {

      $sqlcommand = q{
        SELECT a.nr, a.blz, a.kto, b.lkz
        FROM   acq_liefleit a, acq_adresse b
        WHERE  a.nr = b.nr
      };

   } elsif ( $option eq "get_Supplierdata" ) {

      $sqlcommand = q{
        SELECT nr, gruppe, brgroup, geltung, blz, kto, iban, bic
        FROM   acq_liefleit
      };

   } elsif ( $option eq "count_iban_bic_empty" ) {

      $sqlcommand = q{
        SELECT count(*)
        FROM   acq_liefleit
        WHERE  iban = "" and bic = ""
      };

   } elsif ( $option eq "count_iban_bic_not_empty" ) {

      $sqlcommand = q{
        SELECT count(*)
        FROM   acq_liefleit
        WHERE  iban != "" and bic != ""
      };

   } elsif ( $option eq "set_IBAN_BIC" ) {

      $sqlcommand = q{
        UPDATE acq_liefleit
        SET iban = iban_value,
            bic  = bic_value
        WHERE condition
      };

   } elsif ( $option eq "show_acq_liefleit_record" ) {

      $sqlcommand = q{
        SELECT *
        FROM   acq_liefleit
        WHERE  condition
      };

   } elsif ( $option eq "show_iban" ) {

      $sqlcommand = q{
        SELECT iban
        FROM   acq_liefleit
        WHERE  condition
      };

   } elsif ( $option eq "check_uniqueness" ) {

      $sqlcommand = q{
        SELECT count(*)
        FROM   acq_liefleit
        WHERE  condition
      };

   # ------------------------ #
   # invalid parameter        #
   # ------------------------ #

   } else {

      $sqlcommand  = "invalid parameter for sql command\n\n\n";
      exit;
   }

   if ( $ONLYFORTEST eq "true" ){
      print "... sqlcommand = $sqlcommand\n\n\n";
   }
   return $sqlcommand;
}


###############################################################################
#  Subroutine connect_to_database
###############################################################################


sub connect_to_database {

   # Connect to
   #    Datenbankverwaltung --> SYBASE
   #    Datenbank           --> hffsis
   #    Server              --> sokrates
   #
   # - das Kommando ausfuehren, dass in $SQLCOMMAND steht
   # - evt gibt es noch Parameter fuer das SQL-Kommando;
   # - Abfrageergebnissse werden geschrieben nach @SQLRESULTS;



   if ( $ONLYFORTEST  eq "true" ){
      print ("... running connect_to_database\n\n\n");
   }

   my $counter          = 0;
   my $resultstring     = ();
   my @singlesqlresult  = ();


   # connection to db (DataBaseHandler)
   $dbh = DBI->connect("dbi:Sybase:server=$server;database=$database",
                       $user,
                       $password,
                       { AutoCommit => 1,
                         PrintError => 0,
                         RaiseError => 0
                       }) or die "Fehler bei connect: $DBI::errstr\n";


   # SQL Statement aufbereiten und speichern (Statementhandler)
   $sth = $dbh->prepare( $SQLCOMMAND );
   if ( $ONLYFORTEST  eq "true" ){
      print "... sth = $sth\n\n\n";
   }

   if ( $#_ <  0 ) {


      # ----------------------------------- #
      # SQL Kommando ohne execute-Parameter #
      # ----------------------------------- #

      $sth->execute();

      # alle Abfrage-Ergebnisse in einer Schleife abholen und ausgeben;
      # Ergebnisse werden datensatzweise nach @singlesqlresult geschrieben

      while (@singlesqlresult = $sth->fetchrow_array()) {

         $resultstring = ();
         for ( my $i = 0; $i <= $#singlesqlresult; $i++ ){

            if ( $i == 0 ) { $resultstring = $singlesqlresult[$i];                }
            if ( $i >  0 ) { $resultstring = $resultstring.$SEPARATOR.$singlesqlresult[$i]; }
         }

         push ( @SQLRESULT_ARRAY, $resultstring );
         ### print "... resultstring = $resultstring ($#singlesqlresult)\n";

         $counter ++;
      }
   }


   if ( $#_ >= 0 ) {

      if ( $ONLYFORTEST  eq "true" ){
         print "... running in execute mode ($#_)\n\n\n";
      }

      # ----------------------------------- #
      # SQL Kommando mit execute-Parametern #
      # ----------------------------------- #

      if ( $#_ == 0 ) { $sth->execute( $_[0] );                 }
      if ( $#_ == 1 ) { $sth->execute( $_[0] , $_[1] );         }
      if ( $#_ == 2 ) { $sth->execute( $_[0] , $_[1] , $_[2] ); }

      # alle Abfrage-Ergebnisse in einer Schleife abholen und ausgeben;
      # Ergebnisse werden datensatzweise nach @singlesqlresult geschrieben

      while (@singlesqlresult = $sth->fetchrow_array()) {

         $resultstring = ();
         for ( my $i = 0; $i <= $#singlesqlresult; $i++ ){

            if ( $i == 0 ) { $resultstring = $singlesqlresult[$i];                }
            if ( $i >  0 ) { $resultstring = $resultstring.$SEPARATOR.$singlesqlresult[$i]; }

         }

         push ( @SQLRESULT_ARRAY, $resultstring);
         ### print "... resultstring = $resultstring ($#singlesqlresult)\n";

         $counter ++;
      }
   }


   # ----------------------------------- #
   #                                     #
   # ----------------------------------- #


   # SQL Statement beenden (DB-Verbindung bleibt bestehen)
   $sth->finish();

   # Verbindung zur DB aufheben
   $dbh->disconnect();

   if ( $ONLYFORTEST  eq "true" ){
      print ("\n\n\n");
      print ("... index of highest entry in SQLRESULTARRAY is $#SQLRESULT_ARRAY\n\n\n");
   }
}


###############################################################################
#  Subroutine execute_sql_command
###############################################################################


sub execute_sql_command {


   # - hier erfolgt der Aufruf eines SQL-Kommandos
   # - @SQLRESULT_ARRAY enthaelt das Ergebnis des Aufrufs

   my $msg     = ();
   my $counter = 0;

   if ( $DBAVAILABLE eq "true" ){

      @SQLRESULT_ARRAY = ();
      &connect_to_database();

      if ($ONLYFORTEST eq "true"){
         $counter = $#SQLRESULT_ARRAY + 1;
         print "... $counter entries found in SQLRESULT_ARRAY\n\n\n";
         print "... SQLRESULT_ARRAY = @SQLRESULT_ARRAY\n\n";

         $msg = ();
         $msg = &gen_logfile_string( $INFO, "$counter entries found in SQLRESULT_ARRAY" );
         push ( @LOG_ARRAY, $msg);
      }
   }
}


###############################################################################
#  Subroutine gen_script_info
###############################################################################


sub gen_script_info {

   my $script_status = shift;
   my $about_script  = ();
   my $act_date      = ();

   $act_date = &get_act_date("WITHDATE");
   if ( $script_status eq "start" ){
      $about_script = "$SCRIPTNAME (Version $SCRIPTVERS) started at $act_date";
   } else {
      $about_script = "$SCRIPTNAME (Version $SCRIPTVERS) finished at $act_date";
   }
   print ("\n\n... $about_script\n\n\n");
   $about_script = $INFO.$about_script;
   push ( @LOG_ARRAY, $about_script );
}


###############################################################################
#  Subroutine show_DB_availability
###############################################################################


sub show_DB_availability {

   my $msg        = ();
   my $tmp_string = ();

   if ( $DBAVAILABLE eq "true" ){
             $tmp_string = "working with DB access";
   } else {
             $tmp_string = "working without DB acces";
   }

   $msg = &gen_logfile_string( $INFO, $tmp_string );
   push ( @LOG_ARRAY, $msg);
   print "... $tmp_string\n\n";
}


###############################################################################
#  Subroutine set_helptxt
###############################################################################


sub set_helptxt {


   @HELPTXT=(
   " ",
   " ",
   "-h      gibt eine kurze Beschreibung der Argumentliste",
   " ",

   );
}


###############################################################################
#  Subroutine get_arguments
###############################################################################



sub get_arguments {

   # Description
   #
   #

   my @feld = ();

   my $ONLYFORTEST="true";
   if ( $ONLYFORTEST eq "true" ){

      foreach $var (@ARGV){

         $_=$var;
         s/\s+//gi;

         if ( /-h/ ){

            print "\n\n\n";
            &set_helptxt();
            &signal_array( @HELPTXT );
            exit;

         } elsif( /-help/ ) {

            print "\n\n\n";
            &set_helptxt();
            &signal_array( @HELPTXT );
            exit;

         } else    {

           ###print "... used invalid argument: $var\n\n\n";
           ###exit;
         }
      }
   }
}


################################################################################
#  Subroutine show_error_msg
###############################################################################


sub show_error_msg {


   # - es werden alle Meldungen der Klasse Error aus @LOG_ARRAY herausgefiltert
   # - Errors werden am Ende des Scripts auf Stdout ausgegeben und signalisiert
   # - Warnings und Infos bleiben unberuecksichtigt

   foreach my $var (@LOG_ARRAY){

      $_=$var;
      if ( /$ERROR/gi ){
         push ( @ERROR_ARRAY, $var);
      }
   }


   if ( $#ERROR_ARRAY >= 0 ){

      ###@ERROR_ARRAY = &reduce_redundant_entries_in_array( \@ERROR_ARRAY, "WITHOUT_SORT" );
      my $counter = $#ERROR_ARRAY + 1;

      print"\n\n";
      #print"... $counter errors found:\n\n";
      print"... errors found:\n\n";
      &signal_array( @ERROR_ARRAY );
      print"\n\n";

   } else {

      print"... no errors found\n\n";
   }
}


###############################################################################
#  Subroutine gen_logfile_string
###############################################################################


sub gen_logfile_string {

   # - die subroutine benoetigt gesetzte die Variable $PRINT_TIMESTAMP,
   #   durch die festgelegt wird ob Meldungen einen Zeitstempel erhalten
   #   oder nicht
   # - das Setzen des Zeitstempels kann erhebliche Laufzeit kosten, wenn
   #   es viele Logfile-Eintraege gibt
   # - $msg_type klassifiziert Logfile-Meldungen in drei Gruppen:
   #   Errors    -> werden am Scriptende ausgegeben
   #   Warnings  -> werden am Scriptende nicht ausgegeben
   #   Infos     -> werden am Scriptende nicht ausgegeben

   my $msg_type        = shift;
   my $log_string      = shift;

   if ( $PRINT_TIMESTAMP eq "true" ) {
      my $act_date        = &get_act_date("WITHDATE");
      return $act_date."   ".$msg_type.$log_string;
   } else {
      return $msg_type.$log_string;
   }
}


###############################################################################
#  Subroutine clean_blz_kto
###############################################################################


sub clean_blz_kto {

   # Bereinigung von blz oder kto durch loeschen von
   # Leerzeichen, slash, Punkt und Bindestrich

   $_ = shift;
   s/ //gi;
   s/\///gi;
   s/-//gi;
   s/\.//gi;
   return $_;
}


###############################################################################
#  Subroutine count_IBAN_BIC_records
###############################################################################


sub count_IBAN_BIC_records {

   # Werte fuer tmp_flag:
   # - tmp_flag = before_update   oder
   # - tmp_flag = after_update

   my $tmp_flag = shift;

   if ($DBAVAILABLE eq "true"){

      $SQLCOMMAND=&gen_sql_command( "count_iban_bic_empty" );
      &execute_sql_command();
      if ( $tmp_flag eq "before_update"){
         $IBAN_BIC_NOT_SET_BEFORE_UPDATE = $SQLRESULT_ARRAY[0];
      } else {
         $IBAN_BIC_NOT_SET_AFTER_UPDATE = $SQLRESULT_ARRAY[0];
      }

      $SQLCOMMAND=&gen_sql_command( "count_iban_bic_not_empty" );
      &execute_sql_command();
      if ( $tmp_flag eq "before_update"){
         $IBAN_BIC_SET_BEFORE_UPDATE = $SQLRESULT_ARRAY[0];
      } else {
         $IBAN_BIC_SET_AFTER_UPDATE = $SQLRESULT_ARRAY[0];
      }
   }
}


###############################################################################
#  Subroutine show_counters
###############################################################################


sub show_counters {

   my $msg = ();

   print "... running show_counters\n\n\n";

   $msg = "NUM_IBANRUECK_ENTRIES                          = $NUM_IBANRUECK_ENTRIES";
   $msg = &gen_logfile_string( $INFO, $msg );
   push ( @LOG_ARRAY, $msg );
   print "$msg\n";
   $msg = "NUM_IBANRUECK_ENTRIES_WITHOUT_REDUNDANCE       = $NUM_IBANRUECK_ENTRIES_WITHOUT_REDUNDANCE";
   $msg = &gen_logfile_string( $INFO, $msg );
   push ( @LOG_ARRAY, $msg );
   print "$msg\n";
   $msg = "NUM_IBANRUECK_OK_STATUS                        = $NUM_IBANRUECK_OK_STATUS";
   $msg = &gen_logfile_string( $INFO, $msg );
   push ( @LOG_ARRAY, $msg );
   print "$msg\n";
   $msg = "NUM_IBANRUECK_NOT_OK_STATUS                    = $NUM_IBANRUECK_NOT_OK_STATUS";
   $msg = &gen_logfile_string( $INFO, $msg );
   push ( @LOG_ARRAY, $msg );
   print "$msg\n\n";


   $msg = "NUM_ACQ_LIEFLEIT_ENTRIES                       = $NUM_ACQ_LIEFLEIT_ENTRIES";
   $msg = &gen_logfile_string( $INFO, $msg );
   push ( @LOG_ARRAY, $msg );
   print "$msg\n";
   $msg = "NUM_ACQ_LIEFLEIT_ENTRIES_WITHOUT_REDUNDANCE    = $NUM_ACQ_LIEFLEIT_ENTRIES_WITHOUT_REDUNDANCE";
   $msg = &gen_logfile_string( $INFO, $msg );
   push ( @LOG_ARRAY, $msg );
   print "$msg\n";
   $msg = "NUM_ACQ_LIEFLEIT_ENTRIES_VALID_FORMAT          = $NUM_ACQ_LIEFLEIT_ENTRIES_VALID_FORMAT";
   $msg = &gen_logfile_string( $INFO, $msg );
   push ( @LOG_ARRAY, $msg );
   print "$msg\n";
   $msg = "NUM_ACQ_LIEFLEIT_ENTRIES_INVALID_FORMAT        = $NUM_ACQ_LIEFLEIT_ENTRIES_INVALID_FORMAT";
   $msg = &gen_logfile_string( $INFO, $msg );
   push ( @LOG_ARRAY, $msg );
   print "$msg\n\n";

   $msg = "NUM_BLZ_KTO_STRING_FOUND                       = $NUM_BLZ_KTO_STRING_FOUND";
   $msg = &gen_logfile_string( $INFO, $msg );
   push ( @LOG_ARRAY, $msg );
   print "$msg\n";
   $msg = "NUM_BLZ_KTO_STRING_NOT_FOUND                   = $NUM_BLZ_KTO_STRING_NOT_FOUND";
   $msg = &gen_logfile_string( $INFO, $msg );
   push ( @LOG_ARRAY, $msg );
   print "$msg\n\n";

   print "... counters before and after update\n\n";
   $msg = "IBAN_BIC_SET_BEFORE_UPDATE                     = $IBAN_BIC_SET_BEFORE_UPDATE";
   $msg = &gen_logfile_string( $INFO, $msg );
   push ( @LOG_ARRAY, $msg );
   print "$msg\n";
   $msg = "IBAN_BIC_NOT_SET_BEFORE_UPDATE                 = $IBAN_BIC_NOT_SET_BEFORE_UPDATE";
   $msg = &gen_logfile_string( $INFO, $msg );
   push ( @LOG_ARRAY, $msg );
   print "$msg\n\n";

   if ($DBAVAILABLE eq "true"){
      $msg = "IBAN_BIC_SET_AFTER_UPDATE                      = $IBAN_BIC_SET_AFTER_UPDATE";
      $msg = &gen_logfile_string( $INFO, $msg );
      push ( @LOG_ARRAY, $msg );
      print "$msg\n";
      $msg = "NUM_ACQ_LIEFLEIT_SEPA_RESULTS                  = $NUM_ACQ_LIEFLEIT_SEPA_RESULTS";
      $msg = &gen_logfile_string( $INFO, $msg );
      push ( @LOG_ARRAY, $msg );
      print "$msg\n";
      $msg = "NUM_ACQ_LIEFLEIT_COPY_BLZ_KTO                  = $NUM_ACQ_LIEFLEIT_COPY_BLZ_KTO";
      $msg = &gen_logfile_string( $INFO, $msg );
      push ( @LOG_ARRAY, $msg );
      print "$msg\n";
      ### $msg = "NUM_ACQ_LIEFLEIT_BLZ_KTO_EMPTY                 = $NUM_ACQ_LIEFLEIT_BLZ_KTO_EMPTY";
      ### $msg = &gen_logfile_string( $INFO, $msg );
      ### push ( @LOG_ARRAY, $msg );
      ### print "$msg\n";
      $msg = "IBAN_BIC_NOT_SET_AFTER_UPDATE                  = $IBAN_BIC_NOT_SET_AFTER_UPDATE";
      $msg = &gen_logfile_string( $INFO, $msg );
      push ( @LOG_ARRAY, $msg );
      print "$msg\n";
   }
   push ( @LOG_ARRAY, " " );
   print "\n\n";
}


###############################################################################
#  Subroutine count_error_cases
###############################################################################


sub count_error_cases {

   # basierend auf dem  Endestatus des SEPA Account_Converter
   # eine Fehlerstatistik aufbauen

   my $ende_status  = shift;

   if ( exists $ERROR_HASH{ $ende_status }){
      $ERROR_HASH{$ende_status} ++;
   } else {
      $ERROR_HASH{$ende_status} = 1;
   }
}


###############################################################################
#  Subroutine show_IBANRUECK_error_statistic
###############################################################################


sub show_IBANRUECK_error_statistic {

   # Ausgabe der Fehlerstatistik in folgendem Format:
   # Anzahl des Fehlers  :  Fehlerbeschreibung (Fehlercode)

   my $msg          = ();
   my $extended_msg = ();
   my $separator    = "  :  ";

   print "... running show_IBANRUECK_error_statistic\n\n\n";

   my $ref_hash = shift;
   while (($key, $value) = each(%$ref_hash)){

      if ($key eq "00" ){
         $msg = "Umstellung war erfolgreich oder nicht erforderlich ($key)";
         $extended_msg = &gen_logfile_string( $INFO, $value.$separator.$msg );
         push ( @LOG_ARRAY, $extended_msg);
      } elsif ($key eq "10" ){
         $msg = "Bankleitzahl (Feld 5b) ungueltig ($key)";
         $extended_msg = &gen_logfile_string( $INFO, $value.$separator.$msg );
         push ( @LOG_ARRAY, $extended_msg);
      } elsif ($key eq "11" ){
         $msg = "Aufbau Kontonummer (Feld 5c) falsch, z.B. auf Grund dergemeldeten Pruefziffernrechnung ($key)";
         $extended_msg = &gen_logfile_string( $INFO, $value.$separator.$msg );
         push ( @LOG_ARRAY, $extended_msg);
      } elsif ($key eq "13" ){
         $msg = "Gemeldete Bankleitzahl ist zur Loeschung vorgemerkt und wurdegegen die Nachfolgebankleitzahl ausgetauscht ($key)";
         $extended_msg = &gen_logfile_string( $INFO, $value.$separator.$msg );
         push ( @LOG_ARRAY, $extended_msg);
      } elsif ($key eq "14" ){
         $msg = "IBAN wurde auf Basis einer zur Loeschung vorgemerktenBankleitzahl ermittelt. Es liegt keine Nachfolgebankleitzahl vor ($key)";
         $extended_msg = &gen_logfile_string( $INFO, $value.$separator.$msg );
         push ( @LOG_ARRAY, $extended_msg);
      } elsif (($key eq "15" ) || ($key eq "16") || ($key eq "17") || ($key eq "18") || ($key eq "19")){
         $msg = "Ggf. weitere Fehler basierend auf den Angaben aus Feld 5b undFeld 5c ($key)";
         $extended_msg = &gen_logfile_string( $INFO, $value.$separator.$msg );
         push ( @LOG_ARRAY, $extended_msg);
      } elsif ($key eq "20" ){
         $msg = "Aufbau der IBAN alt (Feld 5) ungueltig ($key)";
         $extended_msg = &gen_logfile_string( $INFO, $value.$separator.$msg );
         push ( @LOG_ARRAY, $extended_msg);
      } elsif ($key eq "21" ){
         $msg = "Pruefziffernrechnung der IBAN alt (Feld 5) falsch ($key)";
         $extended_msg = &gen_logfile_string( $INFO, $value.$separator.$msg );
         push ( @LOG_ARRAY, $extended_msg);
      } elsif ($key eq "22" ){
         $msg = "BIC (Feld 4) ist nicht gueltig ($key)";
         $extended_msg = &gen_logfile_string( $INFO, $value.$separator.$msg );
         push ( @LOG_ARRAY, $extended_msg);
      } elsif ($key eq "2[3-9]" ){
         $msg = "Ggf. weitere Fehler basierend auf Feld 4 und Feld 5 ($key)";
         $extended_msg = &gen_logfile_string( $INFO, $value.$separator.$msg );
         push ( @LOG_ARRAY, $extended_msg);
      } elsif ($key eq "3[0-9]" ){
         $msg = "Ggf. weitere Fehler aus kontenbezogenen Pruefungen ($key)";
         $extended_msg = &gen_logfile_string( $INFO, $value.$separator.$msg );
         push ( @LOG_ARRAY, $extended_msg);
      } elsif ($key eq "40" ){
         $msg = "Konto ist kein Konto der umstellenden Stelle (gem. Feld 5 oder5b) ($key)";
         $extended_msg = &gen_logfile_string( $INFO, $value.$separator.$msg );
         push ( @LOG_ARRAY, $extended_msg);
      } elsif ($key eq "50" ){
         $msg = "IBAN-Berechnung nicht moeglich ($key)";
         $extended_msg = &gen_logfile_string( $INFO, $value.$separator.$msg );
         push ( @LOG_ARRAY, $extended_msg);
      } elsif ($key eq "9[0-9]" ){
         $msg = "Individuelle Fehler und Hinweismeldungen die zwischen dem Einreicher und Bearbeiter der Datei vereinbart sind ($key)";
         $extended_msg = &gen_logfile_string( $INFO, $value.$separator.$msg );
         push( @LOG_ARRAY, $extended_msg);
      } else {
         $msg = "unbekannten SEPA-Errorcode gefunden ($key)";
         $extended_msg = &gen_logfile_string( $INFO, $value.$separator.$msg );
         push( @LOG_ARRAY, $extended_msg);
      }
      printf ("%3d%s%s\n", $value, $separator, $msg);
   }
   push( @LOG_ARRAY, " " );
   print "\n\n\n";
}


###############################################################################
#  Subroutine check_IBANRUECK
###############################################################################


sub check_IBANRUECK {

   # - IBANRUECK einlesen und auf redundante Eintraege ueberpruefen;
   #   redundante Eintraege ins Logfile schreiben
   # - die Ergebnissdatei des SEPA Account Converter pruefen;
   #   Eintraege mit Statuscode ungleich OK zur Recherche ins logfile schreiben
   # - die Felder der Ergebnisdatei IBANRUECK enthalten
   #   feld[0] : Laenderkennzeichen
   #   feld[1] : blz/kto  hintereinander gekettet
   #   feld[2] : BIC/IBAN hintereinander gekettet
   #   feld[3] : Status code
   # - gueltige IBAN/BIC entries werden aussortiert und nach
   #   @IBANRUECK_OK_STATUS_ARRAY geschrieben
   # - der Endestatus wird immer gesetzt

   my @tmp_array        = ();
   my $msg              = ();
   my $tmp_empty_string = " +";
   my $OK_status        = "00";
   my $act_status       = ();

   &write_file_into_array( $IBANRUECK_FILE );
   @IBANRUECK_ARRAY = @INPUT_ARRAY;
   $NUM_IBANRUECK_ENTRIES = $#IBANRUECK_ARRAY + 1;


   @IBANRUECK_WITHOUT_REDUNDANCE_ARRAY = &reduce_redundant_entries_in_array( \@IBANRUECK_ARRAY, "COLLECT_REDUNDANCIES");
   $NUM_IBANRUECK_ENTRIES_WITHOUT_REDUNDANCE = $#IBANRUECK_WITHOUT_REDUNDANCE_ARRAY + 1;

   foreach $var (@IBANRUECK_WITHOUT_REDUNDANCE_ARRAY){

      @tmp_array  = ();
      @tmp_array  = split /$tmp_empty_string/, $var;
      $act_status = ();
      $act_status = $tmp_array[$#tmp_array];

      # Zeilenende loeschen
      $_ = $act_status;
      s/\r//gi;
      s/\n//gi;
      $act_status = $_;

      if ( $act_status eq $OK_status ){

         push ( @IBANRUECK_OK_STATUS_ARRAY, $var );
         $NUM_IBANRUECK_OK_STATUS  ++;

      } else {

         push ( @IBANRUECK_LOGFILE_MSG_ARRAY, $tmp_array[1]." ".$act_status);

         $NUM_IBANRUECK_NOT_OK_STATUS  ++;
         &count_error_cases($act_status);
      }
   }
}


###############################################################################
#  Subroutine expand_IBANRUECK_LOGFILE_MSG_ARRAY
###############################################################################


sub expand_IBANRUECK_LOGFILE_MSG_ARRAY {

   # die Fehlersignalisierung im Logfile von IBANRUECK records mit einem
   # Statuscode ungleich OK wird hier erweitert;
   # zu jedem blz_kto_string werden in der formatierten ACQ_LIEFLEIT_FORMATED
   # alle passenden Lieferanten ermittelt;
   # ein Lieferant ist eindeutig bestimmt durch das Quadrupel
   # (nr, gruppe, brgroup, geltung)

   my $msg          = ();
   my $blz_kto      = ();
   my $statuscode   = ();
   my @tmp_array    = ();
   my $msg_counter  = 0;

   print "... running expand_IBANRUECK_LOGFILE_MSG_ARRAY\n\n\n";

   foreach my $ibanrueck_blz_kto_statuscode_string (@IBANRUECK_LOGFILE_MSG_ARRAY){

      $msg_counter  = 1;
      @tmp_array    = ();
      @tmp_array    = split / /, $ibanrueck_blz_kto_statuscode_string;
      $blz_kto      = $tmp_array[0];
      $statuscode   = $tmp_array[1];

      $msg = ();
      $msg = &gen_logfile_string( $ERROR, "($msg_counter) IBAN-BIC calculation not successful ($blz_kto; Statuscode $statuscode)" );
      push ( @LOG_ARRAY, $msg);

      # avoid metachar of regex in search pattern
      $_ = $blz_kto;
      if ((/\*/gi) || (/\+/gi) || (/\./gi))
      {
         $msg = ();
         $msg = &gen_logfile_string( $INFO, "metachar of regex found in search pattern ($blz_kto)" );
         push ( @LOG_ARRAY, $msg);
	  }
	  else
	  {
         foreach my $acq_liefleit_entry (@ACQ_LIEFLEIT_FORMATED_ARRAY)
		 {
            $_ = $acq_liefleit_entry;
            if ( /$blz_kto/gi )
			{
               $msg_counter ++;
               $msg = ();
               $msg = &gen_logfile_string( $ERROR, "($msg_counter) $acq_liefleit_entry" );
               push ( @LOG_ARRAY, $msg);
            }
         }
      }
      push ( @LOG_ARRAY, " " );
   }
}


###############################################################################
#  Subroutine read_acq_liefleit_entries
###############################################################################


sub read_acq_liefleit_entries {

   # - es werden die folgenden Felder aus allen records von acq_liefleit ausgelesen:
   #   $feld[0] ==> nr
   #   $feld[1] ==> gruppe
   #   $feld[2] ==> brgroup
   #   $feld[3] ==> geltung
   #   $feld[4] ==> blz
   #   $feld[5] ==> kto
   #   $feld[6] ==> iban
   #   $feld[7] ==> bic
   # - es wird gezaehlt, wieviele records es in acq_liefleit gibt, fuer die
   #   beide Felder iban und bic leer sind
   # - in Abhaengigkeit vom Status wird das Ergebnis in unterschiedliche Arrays
   #   geschrieben

   my $status = shift;
   
   if ($DBAVAILABLE eq "true"){

      if ($status eq "before_update"){
      	
         @ACQ_LIEFLEIT_ARRAY = ();
         $SQLCOMMAND=&gen_sql_command( "get_Supplierdata" );
         &execute_sql_command();
         @ACQ_LIEFLEIT_ARRAY = @SQLRESULT_ARRAY; 
             	
         $NUM_ACQ_LIEFLEIT_ENTRIES = $#ACQ_LIEFLEIT_ARRAY + 1;
         
      } else {
      	
      	 @ACQ_LIEFLEIT_AFTER_UPDATE_ARRAY = ();      	 
         $SQLCOMMAND=&gen_sql_command( "get_Supplierdata" );
         &execute_sql_command();      	 
         @ACQ_LIEFLEIT_AFTER_UPDATE_ARRAY = @SQLRESULT_ARRAY;     	
      }

   } else {

      @INPUT_ARRAY = ();
      &write_file_into_array( $ACQ_LIEFLEIT_FILE );
      @ACQ_LIEFLEIT_ARRAY = @INPUT_ARRAY;
      $NUM_ACQ_LIEFLEIT_ENTRIES = $#ACQ_LIEFLEIT_ARRAY + 1;
   }
   if ($status eq "before_update"){
      @ACQ_LIEFLEIT_WITHOUT_REDUNDANCE_ARRAY = &reduce_redundant_entries_in_array( \@ACQ_LIEFLEIT_ARRAY, "COLLECT_REDUNDANCIES");
      $NUM_ACQ_LIEFLEIT_ENTRIES_WITHOUT_REDUNDANCE = $#ACQ_LIEFLEIT_WITHOUT_REDUNDANCE_ARRAY + 1;
   }
}


###############################################################################
#  Subroutine gen_blz_kto_string
###############################################################################


sub gen_blz_kto_string {

   # es wird aus den erneut geholten Lieferantendaten der blz-kto-String
   # aus IBANRUECK nachgebildet mit
   # - Zeichen 1-8      : Bankleitzahl
   # - Zeichen 8-18     : Kontonummer
   # Dieser String kann dann zum Suchen in IBANRUECK benutzt werden, wenn
   # die aktuellen IBAN und BIC Werte dort geholt werden.
   # Eintraege aus ACQ_LIEFLEIT, deren blz_kto_string nicht 18-stellig ist und
   # einem deutschen Format entspricht, werden in ACQ_LIEFLEIT_INVALID_BLZ_KTO_ARRAY
   # gesammelt; Grund: evt. handelt es sich hierbei schon um IBAN, BICs, die bereits
   # in der Vergangenheit in die Felder blz und kto hineingepflegt wurden.
   # Durch den Schalter COPY_BLZ_KTO_TO_IBAN_BIC kann man veranlassen, dass solche
   # Werte nach IBAN und BIC geschrieben werden.

   my $acq_liefleit_record = shift;
   my $blz                 = ();
   my $kto                 = ();
   my $msg                 = ();
   my $tmp_string          = ();
   my $msg_counter         = 1;
   my $blz_string_length   = 8;
   my $kto_string_length   = 10;
   my @tmp_array           = ();


   @tmp_array = split /$SEPARATOR/, $acq_liefleit_record;
   $blz       = $tmp_array[4];
   $kto       = $tmp_array[5];

   # Leerzeichen, slash, Punkt und Bindestrich loeschen
   $blz = clean_blz_kto($blz);
   $kto = clean_blz_kto($kto);

   # ggf. linksbuendig "0" auffuellen
   my $diff = $kto_string_length - length($kto);
   for ( my $counter = 0; $counter < $diff; $counter ++ ){
                     $kto = "0".$kto;
   }

   $tmp_string = $blz.$kto;
   if ( length($tmp_string) != 18 ){

      $msg = ();
      $msg = &gen_logfile_string( $WARNING, "($msg_counter) can not build blz-kto string ($blz - $kto)" );
      push ( @LOG_ARRAY, $msg);
      $msg_counter ++;
      $msg = ();
      $msg = &gen_logfile_string( $WARNING, "($msg_counter) $acq_liefleit_record" );
      push ( @LOG_ARRAY, $msg);
      push ( @LOG_ARRAY, " ");

      push ( @ACQ_LIEFLEIT_INVALID_BLZ_KTO_ARRAY, $acq_liefleit_record );

      $tmp_string = $INVALID;
      $NUM_ACQ_LIEFLEIT_ENTRIES_INVALID_FORMAT ++;

   } else {

      ### print "... build blz-kto string = $tmp_string\n";
   }

   return $tmp_string;
}


###############################################################################
#  Subroutine format_acq_liefleit_array
###############################################################################


sub format_acq_liefleit_array {

   # - blz und kto ueberpruefen, formatieren und zu einem String
   #   zusammenziehen, wie er in IBANRUECK eingetragen ist
   # - mit dem Gemeinschaftsfeld aus blz und kto wird nun ein
   #   fuenfspaltiger Array erzeugt @ACQ_LIEFLEIT_FORMATED_ARRAY

   my $blz_kto_string    = ();
   my $new_record        = ();
   my $blz               = ();
   my $kto               = ();
   my @feld              = ();

   foreach $var (@ACQ_LIEFLEIT_WITHOUT_REDUNDANCE_ARRAY){

      $blz_kto_string = ();
      $new_record     = ();
      $blz            = ();
      $kto            = ();

      @feld           = ();
      @feld           = split /$SEPARATOR/, $var;
      $blz            = $feld[4];
      $kto            = $feld[5];

      ### $blz_kto_string = gen_blz_kto_string( $blz, $kto);
      $blz_kto_string = gen_blz_kto_string( $var );
      if ( $blz_kto_string ne $INVALID ){
         $new_record = $feld[0].$SEPARATOR;
         $new_record = $new_record.$feld[1].$SEPARATOR;
         $new_record = $new_record.$feld[2].$SEPARATOR;
         $new_record = $new_record.$feld[3].$SEPARATOR;
         $new_record = $new_record.$blz_kto_string.$SEPARATOR;
         $new_record = $new_record.$feld[6].$SEPARATOR;
         if ($#feld >= 7 ){ $new_record = $new_record.$feld[7]; }

         push ( @ACQ_LIEFLEIT_FORMATED_ARRAY, $new_record);
      }
   }
   $NUM_ACQ_LIEFLEIT_ENTRIES_VALID_FORMAT = $#ACQ_LIEFLEIT_FORMATED_ARRAY + 1;
}



###############################################################################
#  Subroutine search_blz_kto_string
###############################################################################


sub search_blz_kto_string {

   # zu jedem blz_kto_string aus IBANRUECK_OK_STATUS_ARRAY werden records in
   # @ACQ_LIEFLEIT_FORMATED_ARRAY mit gleichem blz_kto_string gesucht;
   # - blz_kto_string gefunden:
   #   der @ACQ_LIEFLEIT_FORMATED_ARRAY wird in zwei Arrays abgelegt:
   #   @ACQ_LIEFLEIT_SINGLE_HITS_ARRAY sammelt die Eintraege zur Generierung des sql-Kommandos;
   #   @ACQ_LIEFLEIT_ALL_HITS_ARRAY    sammelt alle Treffer und ist ein Kontrollarray, der mit
   #                                   ausgegeben werden kann.
   #
   # - blz_kto_string nicht gefunden:  Rueckgabewert = invalid
   #   der String "invalid" wird nach ACQ_LIEFLEIT_ALL_HITS_ARRAY geschrieben

   my $tmp_blz_kto_string = shift;
   my $msg                = ();
   my $found_counter      = 0;
   my @tmp_array          = ();

   @ACQ_LIEFLEIT_SINGLE_HITS_ARRAY  = ();

   foreach $var (@ACQ_LIEFLEIT_FORMATED_ARRAY){
   	  
   	  @tmp_array = ();
   	  @tmp_array = split /$SEPARATOR/, $var;
   	
      $_ = $tmp_array[4];
      if ( /$tmp_blz_kto_string/gi ){
         $found_counter ++;
         push (@ACQ_LIEFLEIT_ALL_HITS_ARRAY, $var );
         push (@ACQ_LIEFLEIT_SINGLE_HITS_ARRAY, $var );
         $NUM_BLZ_KTO_STRING_FOUND ++;

         # only for test
         # print "($found_counter) $tmp_blz_kto_string => $var\n";
      }
   }

   if ( $found_counter == 0 ){
      $msg = ();
      $msg = &gen_logfile_string( $WARNING, "blz_kto_string not found in ACQ_LIEFLEIT_FORMATED_ARRAY ($tmp_blz_kto_string)" );
      push ( @LOG_ARRAY, $msg);
      $NUM_BLZ_KTO_STRING_NOT_FOUND ++;
   } else {
      if ( $found_counter > 1 ){
         $msg = ();
         $msg = &gen_logfile_string( $INFO, "$found_counter hits in ACQ_LIEFLEIT_FORMATED_ARRAY ($tmp_blz_kto_string)" );
         push ( @LOG_ARRAY, $msg);
      }
   }
}


###############################################################################
#  Subroutine set_IBAN_BIC
###############################################################################


sub set_IBAN_BIC {

   # - zu jedem blz_kto_string aus IBANRUECK_OK_STATUS_ARRAY wird ein
   #   ACQ_LIEFLEIT_FORMATED_ARRAY record mit gleichem blz_kto_string gesucht;
   #   der Ergebnisaustauscharray heisst ACQ_LIEFLEIT_ALL_HITS_ARRAY
   # - IBAN, BIC Existenzflag :
   #   iban_bic_found = true  :  falls IBAN oder BIC oder beide bereits gesetzt sind
   #   iban_bic_found = false :  falls beide Felder nicht gesetzt sind
   # - in welchem Falle IBAN und BIC geschrieben werden:
   #   ist iban_bic_found = false : die aktuell ermittelten Werte fuer IBAN, BIC werden gesetzt
   #   ist iban_bic_found = true  : die aktuell ermittelten Werte fuer IBAN, BIC werden nur gesetzt,
   #                                wenn das Overwrite flag auf true steht
   # - IBAN, BIC Overwrite flag :
   #   $OVERWRITE_IBAN_BIC_FLAG = true  : bestehende Werte duerfen ueberschrieben werden
   #   $OVERWRITE_IBAN_BIC_FLAG = false : bestehende Werte duerfen nicht ueberschrieben werden
   # - IBAN, BIC Eintraege werden via (Lieferanten-)
   #      Nr, Gruppe, brgroup und geltung
   #   in den richtigen record von acq_liefleit adressiert


   my $msg                    = ();
   my $tmp_condition          = ();
   my $tmp_empty_string       = " +";
   my $OK_status              = "00";
   my $iban_bic_found         = ();
   my $counter                = 0;
   my @sepa_tmp_array         = ();
   my $sepa_blz_kto_string    = ();
   my $sepa_iban_bic          = ();
   my @acq_liefleit_tmp_array = ();
   my $acq_liefleit_nr        = ();
   my $acq_liefleit_gruppe    = ();
   my $acq_liefleit_brgroup   = ();
   my $acq_liefleit_geltung   = ();
   my $acq_liefleit_iban      = ();
   my $acq_liefleit_bic       = ();

   print "... running set_IBAN_BIC\n\n\n";

   foreach $var (@IBANRUECK_OK_STATUS_ARRAY){

      @sepa_tmp_array       = ();
      @sepa_tmp_array       = split /$tmp_empty_string/, $var;
      $sepa_blz_kto_string  = ();
      $sepa_blz_kto_string  = $sepa_tmp_array[1];
      $sepa_iban_bic        = ();
      $sepa_iban            = ();
      $sepa_bic             = ();


      # Zeilenende loeschen
      $_ = $var;
      s/\r//gi;
      s/\n//gi;
      $var = $_;


      &search_blz_kto_string($sepa_blz_kto_string);

      $sepa_iban_bic = $sepa_tmp_array[2];
      $sepa_iban     = substr($sepa_iban_bic, 11);
      $sepa_bic      = substr($sepa_iban_bic, 0, 11 );


      for my $formated_acq_liefleit_entry (@ACQ_LIEFLEIT_SINGLE_HITS_ARRAY){

         $counter ++;
         print "... ($counter) iban = $sepa_iban,  bic = $sepa_bic\n";

         $_ = $formated_acq_liefleit_entry;
         if ( /$sepa_blz_kto_string/gi ){

            # --------------------------------- #
            # nr, gruppe, brgroup, geltung      #
            # --------------------------------- #

            @acq_liefleit_tmp_array = ();
            @acq_liefleit_tmp_array = split /$SEPARATOR/, $formated_acq_liefleit_entry;
            $acq_liefleit_nr        = $acq_liefleit_tmp_array[0];
            $acq_liefleit_gruppe    = $acq_liefleit_tmp_array[1];
            $acq_liefleit_brgroup   = $acq_liefleit_tmp_array[2];
            $acq_liefleit_geltung   = $acq_liefleit_tmp_array[3];
            $acq_liefleit_iban      = $acq_liefleit_tmp_array[5];
            $acq_liefleit_bic       = $acq_liefleit_tmp_array[6];

            # --------------------------------- #
            # WHERE condition generieren        #
            # --------------------------------- #

            $tmp_condition = ();
            $tmp_condition = &gen_where_condition( $acq_liefleit_nr,
                                                   $acq_liefleit_gruppe,
                                                   $acq_liefleit_brgroup,
                                                   $acq_liefleit_geltung );

            # --------------------------------- #
            # Eindeutigkeitscheck               #
            # --------------------------------- #

            $SQLCOMMAND=();
            $SQLCOMMAND=&gen_sql_command( "check_uniqueness" );

            $_ = $SQLCOMMAND;
            s/condition/$tmp_condition/gi;
            $SQLCOMMAND = $_;

            if ($DBAVAILABLE eq "true"){
               &execute_sql_command( );
               if ( $SQLRESULT_ARRAY[0] != 1 ){
                  $msg = "WHERE condition is not unique ($tmp_condition)";
                  $msg = &gen_logfile_string( $WARNING, $msg );
                  push ( @LOG_ARRAY, $msg );
                  print "$msg\n\n";
               }
            }

            # --------------------------------- #
            # sql Kommando generieren           #
            # --------------------------------- #

            $SQLCOMMAND=();
            $SQLCOMMAND=&gen_sql_command( "set_IBAN_BIC" );

            $_ = $SQLCOMMAND;
            s/condition/$tmp_condition/gi;
            s/iban_value/\"$sepa_iban\"/gi;
            s/bic_value/\"$sepa_bic\"/gi;
            $SQLCOMMAND = $_;

            if ($ONLYFORTEST eq "true"){
               print "... where condition : $tmp_condition\n";
               print "... SQLCOMMAND      : $SQLCOMMAND\n";
            }

            # --------------------------------- #
            # Schreibbedingung fuer IBAN, BIC   #
            # --------------------------------- #

            $iban_bic_found = "false";

            $_ = $acq_liefleit_iban;
            if (! /^\s*$/ ){ $iban_bic_found = "true"; }
            $_ = $acq_liefleit_bic;
            if (! /^\s*$/ ){ $iban_bic_found = "true"; }

            # --------------------------------- #
            # IBAN, BIC setzen                  #
            # --------------------------------- #

            if ( $iban_bic_found eq "false" ){

               ### Iban, Bic setzen ###
               &execute_sql_command( );
               $NUM_ACQ_LIEFLEIT_SEPA_RESULTS ++;

            } else {

               if ($OVERWRITE_IBAN_BIC_FLAG eq "true"){

                  ### Iban, Bic setzen ###
                  &execute_sql_command( );
                  $NUM_ACQ_LIEFLEIT_SEPA_RESULTS ++;

               } else {

                  $msg = ();
                  $msg = &gen_logfile_string( $INFO, "will not overwrite $formated_acq_liefleit_entry" );
                  push ( @LOG_ARRAY, $msg);
               }
            }
         }
      }
   }
   print "\n\n";  
}


###############################################################################
#  Subroutine write_redundancies_into_logfile
###############################################################################


sub write_redundancies_into_logfile {

   my $ref_array = shift;
   my $msg       = ();

   foreach my $var (@$ref_array){

      $msg = ();
      $msg = &gen_logfile_string( $INFO, "redundancy found ($var)" );
      push ( @LOG_ARRAY, $msg);
   }
}


###############################################################################
#  Subroutine write_blz_kto_to_iban_bic
###############################################################################


sub write_blz_kto_to_iban_bic {

   # - die records aus ACQ_LIEFLEIT mit einem ungueltigen blz_kto_string
   #   stehen in ACQ_LIEFLEIT_INVALID_BLZ_KTO_ARRAY zur Verfuegung.
   #   Die Werte in blz und kto werden extrahiert und nach iban und bic
   #   geschrieben, falls das Kopierflag gesetzt ist.
   # - kopierte blz, kto werden im logfile protokolliert
   # - es macht nur Sinn ibans, bics zu zaehlen, die nicht leer sind;
   #   dieser Fall kommt relativ haeufig vor 

   my $msg               = ();
   my $blz               = ();
   my $kto               = ();
   my $iban              = ();
   my $bic               = ();
   my $iban_bic_found    = ();
   my $blz_kto_empty     = ();
   my $count_condition   = ();
   my @tmp_array         = ();

   if ( $DBAVAILABLE eq "true" ){

      print "... running write_blz_kto_to_iban_bic\n\n\n";

      if ( $COPY_BLZ_KTO_TO_IBAN_BIC eq "true" ){

         $msg = ();
         $msg = &gen_logfile_string( $INFO, "COPY_BLZ_KTO_TO_IBAN_BIC = $COPY_BLZ_KTO_TO_IBAN_BIC" );
         push ( @LOG_ARRAY, $msg);

         foreach $invalid_blz_kto_string (@ACQ_LIEFLEIT_INVALID_BLZ_KTO_ARRAY){

            @tmp_array      = split /$SEPARATOR/, $invalid_blz_kto_string;
            $blz            = $tmp_array[4];
            $kto            = $tmp_array[5];
            $iban           = $tmp_array[6];
            $bic            = $tmp_array[7];

            # --------------------------------- #
            # Bereinigung von blz und kto       #
            # --------------------------------- #

            if ( $CLEAN_BLZ_KTO eq "true" ){
               $blz = clean_blz_kto($blz);
               $kto = clean_blz_kto($kto);
            }

            # --------------------------------- #
            # Schreibbedingung fuer IBAN, BIC   #
            # --------------------------------- #

            $iban_bic_found = "false";

            $_ = $iban;
            if (! /^\s*$/ ){ $iban_bic_found = "true"; }
            $_ = $bic;
            if (! /^\s*$/ ){ $iban_bic_found = "true"; }

            # --------------------------------- #
            # Zaehl-Bedingung                   #
            # --------------------------------- #

            $blz_kto_empty = "false";

            $_ = $blz;
            if (! /^\s*$/ ){ $blz_kto_empty = "true"; }
            $_ = $kto;
            if (! /^\s*$/ ){ $blz_kto_empty = "true"; }

            # --------------------------------- #
            # WHERE condition generieren        #
            # --------------------------------- #

            $tmp_condition = ();
            $tmp_condition = &gen_where_condition( $tmp_array[0],
                                                   $tmp_array[1],
                                                   $tmp_array[2],
                                                   $tmp_array[3] );

            # --------------------------------- #
            # sql Kommando generieren           #
            # --------------------------------- #

            $SQLCOMMAND=&gen_sql_command( "set_IBAN_BIC" );

            $_ = $SQLCOMMAND;
            s/condition/$tmp_condition/gi;
            s/iban_value/\"$kto\"/gi;
            s/bic_value/\"$blz\"/gi;
            $SQLCOMMAND = $_;

            if ($ONLYFORTEST eq "true"){
               print "... where condition : $tmp_condition\n";
               print "... SQLCOMMAND      : $SQLCOMMAND\n";
            }

            # --------------------------------- #
            # IBAN, BIC setzen                  #
            # --------------------------------- #

            if ( $iban_bic_found eq "false" ){

               ### Iban, Bic setzen ###
               &execute_sql_command( );

               $msg = ();
               $msg = &gen_logfile_string( $INFO, "(1) copy (kto, blz) = ($kto, $blz) to (iban, bic)" );
               push ( @LOG_ARRAY, $msg);
               $msg = &gen_logfile_string( $INFO, "(2) $invalid_blz_kto_string" );
               push ( @LOG_ARRAY, $msg);
               push ( @LOG_ARRAY, " " );

               if ($blz_kto_empty eq "false"){
                  $NUM_ACQ_LIEFLEIT_BLZ_KTO_EMPTY ++;	
               } else {
                  $NUM_ACQ_LIEFLEIT_COPY_BLZ_KTO ++;	
               }

            } else {

               if ($OVERWRITE_IBAN_BIC_FLAG eq "true"){

                  ### Iban, Bic setzen ###
                  &execute_sql_command( );

                  $msg = ();
                  $msg = &gen_logfile_string( $INFO, "(1) copy (kto, blz) = ($kto, $blz) to (iban, bic)" );
                  push ( @LOG_ARRAY, $msg);
                  $msg = &gen_logfile_string( $INFO, "(2) $invalid_blz_kto_string" );
                  push ( @LOG_ARRAY, $msg);
                  push ( @LOG_ARRAY, " " );

                  if ($blz_kto_empty eq "false"){
                     $NUM_ACQ_LIEFLEIT_BLZ_KTO_EMPTY ++;	
                  } else {
                     $NUM_ACQ_LIEFLEIT_COPY_BLZ_KTO ++;	
                  }

               } else {

                  $msg = ();
                  $msg = &gen_logfile_string( $INFO, "(1) overwrite flag avoids copy of (blz, kto) to (iban, bic)" );
                  push ( @LOG_ARRAY, $msg);
                  $msg = &gen_logfile_string( $INFO, "(2) $invalid_blz_kto_string" );
                  push ( @LOG_ARRAY, $msg);
                  push ( @LOG_ARRAY, " " );
               }
            }

            # --------------------------------- #
            #                                   #
            # --------------------------------- #

         }
         push ( @LOG_ARRAY, " " );
      }
   }
}


###############################################################################
#  main
###############################################################################


&gen_script_info("start");
&write_array_into_file( $LOGFILE , "" );
&write_array_into_file( "ACQ_LIEFLEIT", "");


# --------------------- #
# running script        #
# --------------------- #


&get_dbname();

&show_DB_availability();

&count_IBAN_BIC_records( "before_update");

&check_IBANRUECK();

&read_acq_liefleit_entries( "before_update" );

&format_acq_liefleit_array();

&expand_IBANRUECK_LOGFILE_MSG_ARRAY();

&set_IBAN_BIC();

&write_blz_kto_to_iban_bic();

&read_acq_liefleit_entries( "after_update" );

&count_IBAN_BIC_records( "after_update");


if ( $LOCALFLAG eq "true" ){

   ### Kontrolldateien ###
   &write_array_into_file( "IBANRUECK_WITHOUT_REDUNDANCE", "" );
   &write_array_into_file( "IBANRUECK_WITHOUT_REDUNDANCE", @IBANRUECK_WITHOUT_REDUNDANCE_ARRAY );
   &write_array_into_file( "IBANRUECK_OK_STATUS", "" );
   &write_array_into_file( "IBANRUECK_OK_STATUS", @IBANRUECK_OK_STATUS_ARRAY );

   if ($DBAVAILABLE eq "true"){
      &write_array_into_file( "ACQ_LIEFLEIT", "" );
      &write_array_into_file( "ACQ_LIEFLEIT", @ACQ_LIEFLEIT_ARRAY );
   }
   &write_array_into_file( "ACQ_LIEFLEIT_WITHOUT_REDUNDANCE", "" );
   &write_array_into_file( "ACQ_LIEFLEIT_WITHOUT_REDUNDANCE", @ACQ_LIEFLEIT_WITHOUT_REDUNDANCE_ARRAY );
   &write_array_into_file( "ACQ_LIEFLEIT_FORMATED", "" );
   &write_array_into_file( "ACQ_LIEFLEIT_FORMATED", @ACQ_LIEFLEIT_FORMATED_ARRAY );
   &write_array_into_file( "ACQ_LIEFLEIT_INVALID_BLZ_KTO", "" );
   &write_array_into_file( "ACQ_LIEFLEIT_INVALID_BLZ_KTO", @ACQ_LIEFLEIT_INVALID_BLZ_KTO_ARRAY );
   &write_array_into_file( "ACQ_LIEFLEIT_ALL_HITS", "" );
   &write_array_into_file( "ACQ_LIEFLEIT_ALL_HITS", @ACQ_LIEFLEIT_ALL_HITS_ARRAY );
   &write_array_into_file( "ACQ_LIEFLEIT_AFTER_UPDATE", "" );
   &write_array_into_file( "ACQ_LIEFLEIT_AFTER_UPDATE", @ACQ_LIEFLEIT_AFTER_UPDATE_ARRAY );
}


&show_error_msg();

&show_IBANRUECK_error_statistic(\%ERROR_HASH);

&show_counters();


# --------------------- #
# avoid warnings        #
# --------------------- #


$INVALID        = ();
$HELPTXT        = ();
$ONLYFORTEST    = ();
$INPUTFILE      = ();
@ERROR_ARRAY    = ();
@RESULT_ARRAY   = ();

$password       = ();
$user           = ();
$server         = ();


###############################################################################
#
###############################################################################


&gen_script_info("end");
&write_array_into_file( $LOGFILE , @LOG_ARRAY );



######################################################################## ENDE #


##HOC######################################################################
#                                                                         #
# HISTORY OF CHANGE:                                         VARIANT: 000 #
#                                                                         #
#----+------+----------------------------------------------+-------+------#
#    | DATE | CHANGES                                      | AUTH  | CNO  #
#    |ddmmyy|                                              |       |      #
#----+------+----------------------------------------------+-------+------#
# 001|131111| Initial creation                             |FDIETZ | 0000 #
#----+------+----------------------------------------------+-------+------#
# 002|250913| avoid metachar of regex in search pattern    |FDIETZ | 0000 #
#----+------+----------------------------------------------+-------+------#
# 003|071013| detect unknown Sepa converter error codes    |FDIETZ | 0000 #
###########################################################################





